""" Contact commands module. """

from gandi.cli.core.base import GandiModule


class Contact(GandiModule):

    """ Module to handle CLI commands."""

    @classmethod
    def info(cls):
        """Display information about a Contact."""
        return cls.call('contact.info')

    @classmethod
    def create(cls, params):
        """Create a new contact."""
        return cls.call('contact.create', params, empty_key=True)

    @classmethod
    def create_dry_run(cls, params):
        """Create a new contact."""
        return cls.call('contact.create', dict(params), empty_key=True,
                        dry_run=True, return_dry_run=True)

    @classmethod
    def balance(cls):
        """Retrieve balance status for a Contact."""
        return cls.call('contact.balance')



from __future__ import print_function
from datetime import datetime


class MockObject(object):

    @classmethod
    def blank_func(cls, *args, **kwargs):
        pass

    @classmethod
    def execute(cls, command, shell=True):
        """ Execute a shell command. """
        if not shell:
            print(' '.join(command))
        else:
            print(command)
        return True

    @classmethod
    def exec_output(cls, command, shell=True, encoding='utf-8'):
        """ Return execution output

        :param encoding: charset used to decode the stdout
        :type encoding: str

        :return: the return of the command
        :rtype: unicode string
        """
        if not shell:
            return ' '.join(command)
        return command

    @classmethod
    def now(cls, *args, **kwargs):
        return datetime(2020, 12, 25, 0, 0, 0)

    @classmethod
    def deprecated(cls, message):
        pass



""" metric module. """

import time
from datetime import datetime

from gandi.cli.core.base import GandiModule


class Metric(GandiModule):

    """ Module to query metrics

    """

    @classmethod
    def query(cls, resources, time_range, query, resource_type, sampler):
        """Query statistics for given resources."""
        if not isinstance(resources, (list, tuple)):
            resources = [resources]

        now = time.time()
        start_utc = datetime.utcfromtimestamp(now - time_range)
        end_utc = datetime.utcfromtimestamp(now)
        date_format = '%Y-%m-%d %H:%M:%S'
        start = start_utc.strftime(date_format)
        end = end_utc.strftime(date_format)
        query = {'start': start,
                 'end': end,
                 'query': query,
                 'resource_id': resources,
                 'resource_type': resource_type,
                 'sampler': sampler}
        return cls.call('hosting.metric.query', query)



# -*- coding: utf-8 -*-
import re

from ..compat import mock
from .base import CommandTestCase
from gandi.cli.commands import certificate


class CertTestCase(CommandTestCase):

    def test_packages(self):

        result = self.invoke_with_exceptions(certificate.packages, [])

        wanted = ("""/!\ "gandi certificate packages" is deprecated.
Please use "gandi certificate plans".
Description            | Name               | Max altnames | Type
-----------------------+--------------------+--------------+-----
Standard Single Domain | cert_std_1_0_0     | 1            | std \
\nStandard Wildcard      | cert_std_w_0_0     | 1            | std \
\nStandard Multi Domain  | cert_std_3_0_0     | 3            | std \
\nStandard Multi Domain  | cert_std_5_0_0     | 5            | std \
\nStandard Multi Domain  | cert_std_10_0_0    | 10           | std \
\nStandard Multi Domain  | cert_std_20_0_0    | 20           | std \
\nPro Single Domain      | cert_pro_1_10_0    | 1            | pro \
\nPro Single Domain      | cert_pro_1_100_0   | 1            | pro \
\nPro Single Domain      | cert_pro_1_100_SGC | 1            | pro \
\nPro Single Domain      | cert_pro_1_250_0   | 1            | pro \
\nPro Wildcard           | cert_pro_w_250_0   | 1            | pro \
\nPro Wildcard           | cert_pro_w_250_SGC | 1            | pro \
\nBusiness Single Domain | cert_bus_1_250_0   | 1            | bus \
\nBusiness Single Domain | cert_bus_1_250_SGC | 1            | bus \
\nBusiness Multi Domain  | cert_bus_3_250_0   | 3            | bus \
\nBusiness Multi Domain  | cert_bus_5_250_0   | 5            | bus \
\nBusiness Multi Domain  | cert_bus_10_250_0  | 10           | bus \
\nBusiness Multi Domain  | cert_bus_20_250_0  | 20           | bus \
\n""")  # noqa

        self.assertEqual(result.output, wanted)

        self.assertEqual(result.exit_code, 0)

    def test_plans(self):

        result = self.invoke_with_exceptions(certificate.plans, [])

        wanted = ("""\
Description            | Max altnames | Type | Warranty
-----------------------+--------------+------+---------
Standard Single Domain | 1            | std  | 0       \
\nStandard Wildcard      | 1            | std  | 0       \
\nStandard Multi Domain  | 3            | std  | 0       \
\nStandard Multi Domain  | 5            | std  | 0       \
\nStandard Multi Domain  | 10           | std  | 0       \
\nStandard Multi Domain  | 20           | std  | 0       \
\nPro Single Domain      | 1            | pro  | 10,000  \
\nPro Single Domain      | 1            | pro  | 100,000 \
\nPro Single Domain      | 1            | pro  | 100,000 \
\nPro Single Domain      | 1            | pro  | 250,000 \
\nPro Wildcard           | 1            | pro  | 250,000 \
\nPro Wildcard           | 1            | pro  | 250,000 \
\nBusiness Single Domain | 1            | bus  | 250,000 \
\nBusiness Single Domain | 1            | bus  | 250,000 \
\nBusiness Multi Domain  | 3            | bus  | 250,000 \
\nBusiness Multi Domain  | 5            | bus  | 250,000 \
\nBusiness Multi Domain  | 10           | bus  | 250,000 \
\nBusiness Multi Domain  | 20           | bus  | 250,000 \
\n""")  # noqa

        self.assertEqual(result.output, wanted)

        self.assertEqual(result.exit_code, 0)

    def test_list(self):

        result = self.invoke_with_exceptions(certificate.list, [])

        self.assertEqual(result.output, """\
cn           : mydomain.name
plan         : Standard Single Domain
----------
cn           : bew.web
plan         : Standard Single Domain
----------
cn           : iheartcli.com
plan         : Pro Single Domain
----------
cn           : cat.lol
plan         : Pro Single Domain
----------
cn           : iheartcli.com
plan         : Business Single Domain
----------
cn           : inter.net
plan         : Business Multi Domain
----------
cn           : lol.cat
plan         : Standard Single Domain
""")
        self.assertEqual(result.exit_code, 0)

    def test_list_all(self):

        args = ['--id', '--status', '--dates', '--altnames', '--csr',
                '--cert']
        result = self.invoke_with_exceptions(certificate.list, args)

        self.assertEqual(result.output, """\
cn           : mydomain.name
plan         : Standard Single Domain
id           : 701
status       : pending
date_created : 20140904T14:06:26
date_end     :
csr          : -----BEGIN CERTIFICATE REQUEST-----
MIICgzCCAWsCAQAwPjERMA8GA1UEAwwIZ2F1dnIuaX...K+I=
-----END CERTIFICATE REQUEST-----
----------
cn           : bew.web
plan         : Standard Single Domain
id           : 771
status       : pending
date_created : 20140904T14:06:26
date_end     :
csr          : -----BEGIN CERTIFICATE REQUEST-----
MIICgzCCAWsCAQAwPjERMA8GA1UEAwwIZ2F1dnIuaX...K+I=
-----END CERTIFICATE REQUEST-----
----------
cn           : iheartcli.com
plan         : Pro Single Domain
id           : 709
status       : valid
date_created : 20140904T14:06:26
date_end     :
csr          : -----BEGIN CERTIFICATE REQUEST-----
MIICgzCCAWsCAQAwPjERMA8GA1UEAwwIZ2F1dnIuaX...K+I=
-----END CERTIFICATE REQUEST-----
----------
cn           : cat.lol
plan         : Pro Single Domain
id           : 709
status       : valid
date_created : 20140904T14:06:26
date_end     :
csr          : -----BEGIN CERTIFICATE REQUEST-----
MIICgzCCAWsCAQAwPjERMA8GA1UEAwwIZ2F1dnIuaX...K+I=
-----END CERTIFICATE REQUEST-----
----------
cn           : iheartcli.com
plan         : Business Single Domain
id           : 769
status       : valid
date_created : 20140904T14:06:26
date_end     :
csr          : -----BEGIN CERTIFICATE REQUEST-----
MIICgzCCAWsCAQAwPjERMA8GA1UEAwwIZ2F1dnIuaX...K+I=
-----END CERTIFICATE REQUEST-----
----------
cn           : inter.net
plan         : Business Multi Domain
id           : 706
status       : valid
date_created : 20140904T14:06:26
date_end     :
csr          : -----BEGIN CERTIFICATE REQUEST-----
MIICgzCCAWsCAQAwPjERMA8GA1UEAwwIZ2F1dnIuaX...K+I=
-----END CERTIFICATE REQUEST-----
----------
cn           : lol.cat
plan         : Standard Single Domain
id           : 710
status       : valid
date_created : 20150318T00:00:00
date_end     : 20160318T00:00:00
csr          : -----BEGIN CERTIFICATE REQUEST-----
MIICgzCCAWsCAQAwPjERMA8GA1UEAwwIZ2F1dnIuaX...K+I=
-----END CERTIFICATE REQUEST-----
cert         : \
\n-----BEGIN CERTIFICATE-----
MIIE5zCCA8+gAwIBAgIQAkC4TU9JG8wqhf4FCrsNGTANBgkqhkiG9w0BAQsFADBf
MQswCQYDVQQGEwJGUjEOMAwGA1UECBMFUGFyaXMxDj...tU6XzbS6/s2D1/N1wWO
OCD/V3XAROtKr1a0mtJ8n7SZyzr0j3weRbN7nV24RDQ6d4+GHy5zZstKyDrTknlu
yyZuDAAYAQJ+nrL5p1gxVNwj1f5XKFk=
-----END CERTIFICATE-----
altname      : pouet.lol.cat
""")
        self.assertEqual(result.exit_code, 0)

    def test_info(self):
        args = ['inter.net', 'bew.web']
        result = self.invoke_with_exceptions(certificate.info, args)

        self.assertEqual(result.output, """cn           : inter.net
date_created : 20140904T14:06:26
date_end     :
plan         : Business Multi Domain
status       : valid
----------
cn           : bew.web
date_created : 20140904T14:06:26
date_end     :
plan         : Standard Single Domain
status       : pending
""")
        self.assertEqual(result.exit_code, 0)

    def test_info_all(self):

        args = ['inter.net', '--id', '--altnames', '--csr', '--cert']
        result = self.invoke_with_exceptions(certificate.info, args)

        self.assertEqual(result.output, """cn           : inter.net
date_created : 20140904T14:06:26
date_end     :
plan         : Business Multi Domain
status       : valid
id           : 706
csr          : -----BEGIN CERTIFICATE REQUEST-----
MIICgzCCAWsCAQAwPjERMA8GA1UEAwwIZ2F1dnIuaX...K+I=
-----END CERTIFICATE REQUEST-----
""")
        self.assertEqual(result.exit_code, 0)

    def test_create(self):
        csr = '''-----BEGIN CERTIFICATE REQUEST-----
MIICWjCCAUICAQAwFTETMBEGA1UEAwwKZG9tYWluLnRsZDCCASIwDQYJKoZIhvcN
AQEBBQADggEPADCCAQoCggEBAKYPfDoiWuWDwJb+fZhOHA++9yYy1BbxnY729hSd
/P12kw1HeIL5CGIhZLpJrwRQmLPTlJ0VttFaqpNm7mEISr+GMJzEWBTyD8750hbW
bXwZBcsWi8AsOsnT+sh/cTKGlJctA346HKU3tLlZsvI4ecfnlIZk5Yefgf+78abz
SzSV47gPDUNQvGIzP9QPE4bEFu5NjdxPg3ylaQ5cv8iiWHn4iUCRXlxxNfHmH7xE
ysFlsD6KnKjR5eYLKBcATeqopGPi72KlcDn5lmtdWsd9aGSl5KlkKQC497buqjbr
H31lMAGAC7At6S7AF5GIT5KGjN6KyPrzUOn7FrhNUcnpUQMCAwEAAaAAMA0GCSqG
SIb3DQEBCwUAA4IBAQCBM6wc9DfsI1htRhAz7/RfOIn7kb6LygOSEgfb757My+60
N/WP9ndpmob0PW18B1vXBloZEkO/aNTXCGAIPJXRkeTYVhEE2B7K3pc9IiNmLxXC
3b2cwUjgmNw9wmFZ4AuHqzWHevqix3m7Acpkl5ugcCsTVOX3mx84MSguSC+5AWfm
DG0VmOWZ0tWjyZuKgtoXgHnH3whEac+pM7M3J+z94/msO9hnpUOQNt4XALEoONrv
+xE1FDGhRJAx9AYOtTBQSFLqKB4D6W2hhDVLirxQuJ/lC/l8tyEu96ggfDRrMXE4
v0L9Vc0443fop+UbFCabF0NWM6rJ31Nlv7s3mQIA
-----END CERTIFICATE REQUEST-----'''

        result = self.invoke_with_exceptions(certificate.create,
                                             ['--csr', csr,
                                              '--duration', 2,
                                              '--max-altname', '5',
                                              ])

        wanted = '''The certificate create operation is 1
You can follow it with:
$ gandi certificate follow 1
When the operation is DONE, you can retrieve the .crt with:
$ gandi certificate export "domain.tld"
'''

        self.assertEqual(result.output, wanted)
        self.assertEqual(result.exit_code, 0)

    def test_create_package_deprecated(self):
        csr = '''-----BEGIN CERTIFICATE REQUEST-----
MIICWjCCAUICAQAwFTETMBEGA1UEAwwKZG9tYWluLnRsZDCCASIwDQYJKoZIhvcN
AQEBBQADggEPADCCAQoCggEBAKYPfDoiWuWDwJb+fZhOHA++9yYy1BbxnY729hSd
/P12kw1HeIL5CGIhZLpJrwRQmLPTlJ0VttFaqpNm7mEISr+GMJzEWBTyD8750hbW
bXwZBcsWi8AsOsnT+sh/cTKGlJctA346HKU3tLlZsvI4ecfnlIZk5Yefgf+78abz
SzSV47gPDUNQvGIzP9QPE4bEFu5NjdxPg3ylaQ5cv8iiWHn4iUCRXlxxNfHmH7xE
ysFlsD6KnKjR5eYLKBcATeqopGPi72KlcDn5lmtdWsd9aGSl5KlkKQC497buqjbr
H31lMAGAC7At6S7AF5GIT5KGjN6KyPrzUOn7FrhNUcnpUQMCAwEAAaAAMA0GCSqG
SIb3DQEBCwUAA4IBAQCBM6wc9DfsI1htRhAz7/RfOIn7kb6LygOSEgfb757My+60
N/WP9ndpmob0PW18B1vXBloZEkO/aNTXCGAIPJXRkeTYVhEE2B7K3pc9IiNmLxXC
3b2cwUjgmNw9wmFZ4AuHqzWHevqix3m7Acpkl5ugcCsTVOX3mx84MSguSC+5AWfm
DG0VmOWZ0tWjyZuKgtoXgHnH3whEac+pM7M3J+z94/msO9hnpUOQNt4XALEoONrv
+xE1FDGhRJAx9AYOtTBQSFLqKB4D6W2hhDVLirxQuJ/lC/l8tyEu96ggfDRrMXE4
v0L9Vc0443fop+UbFCabF0NWM6rJ31Nlv7s3mQIA
-----END CERTIFICATE REQUEST-----'''

        result = self.invoke_with_exceptions(certificate.create,
                                             ['--csr', csr,
                                              '--duration', 2,
                                              '--package', 'cert_std_1_0_0',
                                              ])

        wanted = '''\
/!\\ Using --package is deprecated, please replace it by --type (in std, pro \
or bus) and --max-altname to set the max number of altnames.
The certificate create operation is 1
You can follow it with:
$ gandi certificate follow 1
When the operation is DONE, you can retrieve the .crt with:
$ gandi certificate export "domain.tld"
'''

        self.assertEqual(result.output, wanted)
        self.assertEqual(result.exit_code, 0)

    def test_create_no_csr(self):
        result = self.invoke_with_exceptions(certificate.create,
                                             ['--duration', 2,
                                              '--max-altname', '5',
                                              ])

        wanted = """You need a CSR or a CN to create a certificate.
"""

        self.assertEqual(result.output, wanted)
        self.assertEqual(result.exit_code, 0)

    def test_create_bad_csr(self):
        args = ['--csr', 'badpath.csr', '--pk', 'badpath.key',
                '--package', 'cert_std_3_0_0', '--dcv-method', 'email']
        result = self.invoke_with_exceptions(certificate.create, args)
        wanted = """Unable to parse provided csr: badpath.csr
"""
        self.assertEqual(result.output, wanted)
        self.assertEqual(result.exit_code, 0)

    def test_create_no_package_and_option(self):
        csr = '''-----BEGIN CERTIFICATE REQUEST-----
MIICWjCCAUICAQAwFTETMBEGA1UEAwwKZG9tYWluLnRsZDCCASIwDQYJKoZIhvcN
AQEBBQADggEPADCCAQoCggEBAKYPfDoiWuWDwJb+fZhOHA++9yYy1BbxnY729hSd
/P12kw1HeIL5CGIhZLpJrwRQmLPTlJ0VttFaqpNm7mEISr+GMJzEWBTyD8750hbW
bXwZBcsWi8AsOsnT+sh/cTKGlJctA346HKU3tLlZsvI4ecfnlIZk5Yefgf+78abz
SzSV47gPDUNQvGIzP9QPE4bEFu5NjdxPg3ylaQ5cv8iiWHn4iUCRXlxxNfHmH7xE
ysFlsD6KnKjR5eYLKBcATeqopGPi72KlcDn5lmtdWsd9aGSl5KlkKQC497buqjbr
H31lMAGAC7At6S7AF5GIT5KGjN6KyPrzUOn7FrhNUcnpUQMCAwEAAaAAMA0GCSqG
SIb3DQEBCwUAA4IBAQCBM6wc9DfsI1htRhAz7/RfOIn7kb6LygOSEgfb757My+60
N/WP9ndpmob0PW18B1vXBloZEkO/aNTXCGAIPJXRkeTYVhEE2B7K3pc9IiNmLxXC
3b2cwUjgmNw9wmFZ4AuHqzWHevqix3m7Acpkl5ugcCsTVOX3mx84MSguSC+5AWfm
DG0VmOWZ0tWjyZuKgtoXgHnH3whEac+pM7M3J+z94/msO9hnpUOQNt4XALEoONrv
+xE1FDGhRJAx9AYOtTBQSFLqKB4D6W2hhDVLirxQuJ/lC/l8tyEu96ggfDRrMXE4
v0L9Vc0443fop+UbFCabF0NWM6rJ31Nlv7s3mQIA
-----END CERTIFICATE REQUEST-----'''

        result = self.invoke_with_exceptions(certificate.create,
                                             ['--csr', csr,
                                              '--duration', 2,
                                              '--max-altname', '5',
                                              '--package', 'cert_std_1_0_0',
                                              ])

        wanted = """Please do not use --package at the same time you use \
--type, --max-altname or --warranty.
"""

        self.assertEqual(result.output, wanted)
        self.assertEqual(result.exit_code, 0)

    def test_create_warranty(self):
        csr = '''-----BEGIN CERTIFICATE REQUEST-----
MIICWjCCAUICAQAwFTETMBEGA1UEAwwKZG9tYWluLnRsZDCCASIwDQYJKoZIhvcN
AQEBBQADggEPADCCAQoCggEBAKYPfDoiWuWDwJb+fZhOHA++9yYy1BbxnY729hSd
/P12kw1HeIL5CGIhZLpJrwRQmLPTlJ0VttFaqpNm7mEISr+GMJzEWBTyD8750hbW
bXwZBcsWi8AsOsnT+sh/cTKGlJctA346HKU3tLlZsvI4ecfnlIZk5Yefgf+78abz
SzSV47gPDUNQvGIzP9QPE4bEFu5NjdxPg3ylaQ5cv8iiWHn4iUCRXlxxNfHmH7xE
ysFlsD6KnKjR5eYLKBcATeqopGPi72KlcDn5lmtdWsd9aGSl5KlkKQC497buqjbr
H31lMAGAC7At6S7AF5GIT5KGjN6KyPrzUOn7FrhNUcnpUQMCAwEAAaAAMA0GCSqG
SIb3DQEBCwUAA4IBAQCBM6wc9DfsI1htRhAz7/RfOIn7kb6LygOSEgfb757My+60
N/WP9ndpmob0PW18B1vXBloZEkO/aNTXCGAIPJXRkeTYVhEE2B7K3pc9IiNmLxXC
3b2cwUjgmNw9wmFZ4AuHqzWHevqix3m7Acpkl5ugcCsTVOX3mx84MSguSC+5AWfm
DG0VmOWZ0tWjyZuKgtoXgHnH3whEac+pM7M3J+z94/msO9hnpUOQNt4XALEoONrv
+xE1FDGhRJAx9AYOtTBQSFLqKB4D6W2hhDVLirxQuJ/lC/l8tyEu96ggfDRrMXE4
v0L9Vc0443fop+UbFCabF0NWM6rJ31Nlv7s3mQIA
-----END CERTIFICATE REQUEST-----'''

        result = self.invoke_with_exceptions(certificate.create,
                                             ['--csr', csr,
                                              '--duration', 2,
                                              '--max-altname', '5',
                                              '--type', 'std',
                                              '--warranty', '250',
                                              ])

        wanted = """The warranty can only be specified for pro certificates.
"""

        self.assertEqual(result.output, wanted)
        self.assertEqual(result.exit_code, 0)

    def test_create_no_package(self):
        csr = '''-----BEGIN CERTIFICATE REQUEST-----
MIICWjCCAUICAQAwFTETMBEGA1UEAwwKZG9tYWluLnRsZDCCASIwDQYJKoZIhvcN
AQEBBQADggEPADCCAQoCggEBAKYPfDoiWuWDwJb+fZhOHA++9yYy1BbxnY729hSd
/P12kw1HeIL5CGIhZLpJrwRQmLPTlJ0VttFaqpNm7mEISr+GMJzEWBTyD8750hbW
bXwZBcsWi8AsOsnT+sh/cTKGlJctA346HKU3tLlZsvI4ecfnlIZk5Yefgf+78abz
SzSV47gPDUNQvGIzP9QPE4bEFu5NjdxPg3ylaQ5cv8iiWHn4iUCRXlxxNfHmH7xE
ysFlsD6KnKjR5eYLKBcATeqopGPi72KlcDn5lmtdWsd9aGSl5KlkKQC497buqjbr
H31lMAGAC7At6S7AF5GIT5KGjN6KyPrzUOn7FrhNUcnpUQMCAwEAAaAAMA0GCSqG
SIb3DQEBCwUAA4IBAQCBM6wc9DfsI1htRhAz7/RfOIn7kb6LygOSEgfb757My+60
N/WP9ndpmob0PW18B1vXBloZEkO/aNTXCGAIPJXRkeTYVhEE2B7K3pc9IiNmLxXC
3b2cwUjgmNw9wmFZ4AuHqzWHevqix3m7Acpkl5ugcCsTVOX3mx84MSguSC+5AWfm
DG0VmOWZ0tWjyZuKgtoXgHnH3whEac+pM7M3J+z94/msO9hnpUOQNt4XALEoONrv
+xE1FDGhRJAx9AYOtTBQSFLqKB4D6W2hhDVLirxQuJ/lC/l8tyEu96ggfDRrMXE4
v0L9Vc0443fop+UbFCabF0NWM6rJ31Nlv7s3mQIA
-----END CERTIFICATE REQUEST-----'''

        result = self.invoke_with_exceptions(certificate.create,
                                             ['--csr', csr,
                                              '--duration', 2,
                                              '--max-altname', '5',
                                              '--cn', '*.lol.cat',
                                              '--altnames', 'pouet.lol.cat',
                                              '--altnames', 'grumpf.lol.cat',
                                              ])

        wanted = """\
You can't have a wildcard with multidomain certificate.
"""

        self.assertEqual(result.output, wanted)
        self.assertEqual(result.exit_code, 0)

    def test_create_csr_empty(self):
        args = ['--csr', 'sandbox/example.txt',
                '--duration', 2,
                '--max-altname', '5',
                '--cn', '*.lol.cat',
                '--altnames', 'pouet.lol.cat',
                '--altnames', 'grumpf.lol.cat',
                ]
        result = self.isolated_invoke_with_exceptions(certificate.create,
                                                      args,
                                                      temp_content='')

        self.assertEqual(result.output, '')
        self.assertEqual(result.exit_code, 0)

    def test_create_wild_altnames(self):
        csr = '''-----BEGIN CERTIFICATE REQUEST-----
MIICWjCCAUICAQAwFTETMBEGA1UEAwwKZG9tYWluLnRsZDCCASIwDQYJKoZIhvcN
AQEBBQADggEPADCCAQoCggEBAKYPfDoiWuWDwJb+fZhOHA++9yYy1BbxnY729hSd
/P12kw1HeIL5CGIhZLpJrwRQmLPTlJ0VttFaqpNm7mEISr+GMJzEWBTyD8750hbW
bXwZBcsWi8AsOsnT+sh/cTKGlJctA346HKU3tLlZsvI4ecfnlIZk5Yefgf+78abz
SzSV47gPDUNQvGIzP9QPE4bEFu5NjdxPg3ylaQ5cv8iiWHn4iUCRXlxxNfHmH7xE
ysFlsD6KnKjR5eYLKBcATeqopGPi72KlcDn5lmtdWsd9aGSl5KlkKQC497buqjbr
H31lMAGAC7At6S7AF5GIT5KGjN6KyPrzUOn7FrhNUcnpUQMCAwEAAaAAMA0GCSqG
SIb3DQEBCwUAA4IBAQCBM6wc9DfsI1htRhAz7/RfOIn7kb6LygOSEgfb757My+60
N/WP9ndpmob0PW18B1vXBloZEkO/aNTXCGAIPJXRkeTYVhEE2B7K3pc9IiNmLxXC
3b2cwUjgmNw9wmFZ4AuHqzWHevqix3m7Acpkl5ugcCsTVOX3mx84MSguSC+5AWfm
DG0VmOWZ0tWjyZuKgtoXgHnH3whEac+pM7M3J+z94/msO9hnpUOQNt4XALEoONrv
+xE1FDGhRJAx9AYOtTBQSFLqKB4D6W2hhDVLirxQuJ/lC/l8tyEu96ggfDRrMXE4
v0L9Vc0443fop+UbFCabF0NWM6rJ31Nlv7s3mQIA
-----END CERTIFICATE REQUEST-----'''

        result = self.invoke_with_exceptions(certificate.create,
                                             ['--csr', csr,
                                              '--duration', 2,
                                              '--max-altname', '5',
                                              '--type', 'pro',
                                              '--warranty', '250',
                                              ])

        wanted = """\
Can't find any plan with your params.
Please call : "gandi certificate plans".
"""

        self.assertEqual(result.output, wanted)
        self.assertEqual(result.exit_code, 0)

    def test_export_ok(self):
        args = ['lol.cat']
        with mock.patch('gandi.cli.commands.certificate.open',
                        create=True) as mock_open:
            mock_open.return_value = mock.MagicMock()

            result = self.invoke_with_exceptions(certificate.export, args)

        wanted = """wrote lol.cat.crt
"""
        self.assertEqual(result.output, wanted)
        self.assertEqual(result.exit_code, 0)

    def test_export_no_cert(self):
        args = ['bew.web']
        with mock.patch('gandi.cli.commands.certificate.open',
                        create=True) as mock_open:
            mock_open.return_value = mock.MagicMock()

            result = self.invoke_with_exceptions(certificate.export, args)

        self.assertEqual(result.output, '')
        self.assertEqual(result.exit_code, 0)

    def test_export_invalid(self):
        args = ['mydomain.name']
        with mock.patch('gandi.cli.commands.certificate.open',
                        create=True) as mock_open:
            mock_open.return_value = mock.MagicMock()

            result = self.invoke_with_exceptions(certificate.export, args)

        self.assertEqual(result.output, """\
The certificate must be in valid status to be exported (701).
""")
        self.assertEqual(result.exit_code, 0)

    def test_export_multiple_ok(self):
        args = ['lol.cat', 'inter.net', '-o', 'pouet.crt']
        with mock.patch('gandi.cli.commands.certificate.open',
                        create=True) as mock_open:
            mock_open.return_value = mock.MagicMock()

            result = self.invoke_with_exceptions(certificate.export, args)

        wanted = """\
Too many certs found, you must specify which cert you want to export
"""
        self.assertEqual(result.output, wanted)
        self.assertEqual(result.exit_code, 0)

    def test_export_exists(self):
        args = ['lol.cat']
        with mock.patch('gandi.cli.commands.certificate.os.path.isfile',
                        create=True) as mock_isfile:
            mock_isfile.return_value = True

            result = self.invoke_with_exceptions(certificate.export, args)

        wanted = """The file lol.cat.crt already exists.
"""
        self.assertEqual(result.output, wanted)
        self.assertEqual(result.exit_code, 0)

    def test_export_nothing(self):
        args = ['inter.net']
        result = self.invoke_with_exceptions(certificate.export, args)

        self.assertEqual(result.output, '')
        self.assertEqual(result.exit_code, 0)

    def test_export_intermediate_business_ok(self):
        args = ['inter.net', '-i']
        with mock.patch('gandi.cli.commands.certificate.open',
                        create=True) as mock_open:
            mock_open.return_value = mock.MagicMock()

            with mock.patch('gandi.cli.commands.certificate.requests.get',
                            create=True) as mock_get:
                mock_get.return_value = mock.MagicMock()

                result = self.invoke_with_exceptions(certificate.export, args)

        wanted = """Business certs do not need intermediates.
"""
        self.assertEqual(result.output, wanted)
        self.assertEqual(result.exit_code, 0)

    def test_export_intermediate_ok(self):
        args = ['lol.cat', '-i']
        with mock.patch('gandi.cli.commands.certificate.open',
                        create=True) as mock_open:
            mock_open.return_value = mock.MagicMock()

            with mock.patch('gandi.cli.commands.certificate.requests.get',
                            create=True) as mock_get:
                mock_get.return_value = mock.MagicMock()

                result = self.invoke_with_exceptions(certificate.export, args)

        wanted = """wrote lol.cat.crt
wrote lol.cat.inter.crt
"""
        self.assertEqual(result.output, wanted)
        self.assertEqual(result.exit_code, 0)

    def test_export_intermediate_sgc_ok(self):
        args = ['cat.lol', '-i']
        with mock.patch('gandi.cli.commands.certificate.open',
                        create=True) as mock_open:
            mock_open.return_value = mock.MagicMock()

            with mock.patch('gandi.cli.commands.certificate.requests.get',
                            create=True) as mock_get:
                mock_get.return_value = mock.MagicMock()

                result = self.invoke_with_exceptions(certificate.export, args)

        wanted = """wrote cat.lol.inter.crt
"""
        self.assertEqual(result.output, wanted)
        self.assertEqual(result.exit_code, 0)

    def test_update(self):
        args = ['inter.net']
        result = self.invoke_with_exceptions(certificate.update, args)

        wanted = """\
openssl req -new -newkey rsa:2048 -sha256 -nodes -out inter.net.csr \
-keyout inter.net.key -subj "/CN=inter.net"
The certificate update operation is 400
You can follow it with:
$ gandi certificate follow 400
When the operation is DONE, you can retrieve the .crt with:
$ gandi certificate export "inter.net"
"""
        self.assertEqual(result.output, wanted)
        self.assertEqual(result.exit_code, 0)

    def test_update_multi(self):
        args = ['iheartcli.com']
        result = self.invoke_with_exceptions(certificate.update, args)

        wanted = """\
Will not update, iheartcli.com is not precise enough.
  * cert : 709
  * cert : 769
"""
        self.assertEqual(result.output, wanted)
        self.assertEqual(result.exit_code, 0)

    def test_follow(self):
        args = ['600']
        result = self.invoke_with_exceptions(certificate.follow, args)

        wanted = """\
type        : certificate_update
step        : DONE
"""
        self.assertEqual(result.output, wanted)
        self.assertEqual(result.exit_code, 0)

    def test_change_dcv(self):
        args = ['lol.cat', '--dcv-method', 'dns']
        result = self.invoke_with_exceptions(certificate.change_dcv, args)

        wanted = """\
You have to add these records in your domain zone :
920F78CCE11DA7D9554.lol.cat. 10800 IN CNAME AD6A9D35FF5BD9FB03A41.comodoca.com.
"""
        self.assertEqual(result.output, wanted)
        self.assertEqual(result.exit_code, 0)

    def test_change_dcv_not_valid(self):
        args = ['mydomain.name', '--dcv-method', 'dns']
        result = self.invoke_with_exceptions(certificate.change_dcv, args)

        wanted = """\
This certificate operation is not in the good step to update the DCV method.
"""
        self.assertEqual(result.output, wanted)
        self.assertEqual(result.exit_code, 0)

    def test_change_dcv_multi(self):
        args = ['iheartcli.com', '--dcv-method', 'dns']
        result = self.invoke_with_exceptions(certificate.change_dcv, args)

        wanted = """\
Will not update, iheartcli.com is not precise enough.
  * cert : 709
  * cert : 769
"""
        self.assertEqual(result.output, wanted)
        self.assertEqual(result.exit_code, 0)

    def test_change_dcv_no_oper(self):
        args = ['cat.lol', '--dcv-method', 'dns']
        result = self.invoke_with_exceptions(certificate.change_dcv, args)

        wanted = """\
Can not find any operation for this certificate.
"""
        self.assertEqual(result.output, wanted)
        self.assertEqual(result.exit_code, 0)

    def test_resend_dcv(self):
        args = ['lol.cat']
        result = self.invoke_with_exceptions(certificate.resend_dcv, args)

        self.assertEqual(result.output, '')
        self.assertEqual(result.exit_code, 0)

    def test_resend_dcv_multi(self):
        args = ['iheartcli.com']
        result = self.invoke_with_exceptions(certificate.resend_dcv, args)

        wanted = """\
Will not update, iheartcli.com is not precise enough.
  * cert : 709
  * cert : 769
"""
        self.assertEqual(result.output, wanted)
        self.assertEqual(result.exit_code, 0)

    def test_resend_dcv_no_oper(self):
        args = ['cat.lol']
        result = self.invoke_with_exceptions(certificate.resend_dcv, args)

        self.assertEqual(result.output, """\
Can not find any operation for this certificate.
""")
        self.assertEqual(result.exit_code, 0)

    def test_resend_dcv_not_email(self):
        args = ['inter.net']
        result = self.invoke_with_exceptions(certificate.resend_dcv, args)

        self.assertEqual(result.output, """\
This certificate operation is not in email DCV.
""")
        self.assertEqual(result.exit_code, 0)

    def test_resend_dcv_not_valid(self):
        args = ['mydomain.name']
        result = self.invoke_with_exceptions(certificate.resend_dcv, args)

        self.assertEqual(result.output, """\
This certificate operation is not in the good step to resend the DCV.
""")
        self.assertEqual(result.exit_code, 0)

    def test_delete_force(self):
        args = ['lol.cat', '--force']
        result = self.invoke_with_exceptions(certificate.delete, args)

        output = re.sub(r'\[#+\]', '[###]', result.output.strip())

        self.assertEqual(output, """\
Deleting your certificate.
\rProgress: [###] 100.00%  00:00:00  \
\nYour certificate 710 has been deleted.""")
        self.assertEqual(result.exit_code, 0)

    def test_delete_multiple(self):
        args = ['iheartcli.com', '--force']
        result = self.invoke_with_exceptions(certificate.delete, args)

        output = re.sub(r'\[#+\]', '[###]', result.output.strip())

        self.assertEqual(output, """\
Will not delete, iheartcli.com is not precise enough.
  * cert : 709
  * cert : 769""")
        self.assertEqual(result.exit_code, 0)

    def test_delete_prompt_ok(self):
        args = ['lol.cat']
        result = self.invoke_with_exceptions(certificate.delete, args,
                                             input='y\n')

        output = re.sub(r'\[#+\]', '[###]', result.output.strip())

        self.assertEqual(output, """\
Are you sure to delete the certificate lol.cat? [y/N]: y
Deleting your certificate.
\rProgress: [###] 100.00%  00:00:00  \
\nYour certificate 710 has been deleted.""")
        self.assertEqual(result.exit_code, 0)

    def test_delete_prompt_ko(self):
        args = ['lol.cat']
        result = self.invoke_with_exceptions(certificate.delete, args,
                                             input='N\n')

        output = re.sub(r'\[#+\]', '[###]', result.output.strip())

        self.assertEqual(output, """\
Are you sure to delete the certificate lol.cat? [y/N]: N""")
        self.assertEqual(result.exit_code, 0)



""" Hosted certificate commands module. """

import os
from gandi.cli.core.base import GandiModule


class HostedCert(GandiModule):
    """ Module to handle CLI commands.

    $ gandi certstore list
    $ gandi certstore info
    $ gandi certstore create
    $ gandi certstore delete
    """

    @classmethod
    def from_fqdn(cls, fqdn):
        return cls.list({'fqdns': '%s' % fqdn, 'state': 'created'})

    @classmethod
    def usable_id(cls, id):
        """ Retrieve id from single input. """
        hcs = cls.from_fqdn(id)
        if hcs:
            return [hc_['id'] for hc_ in hcs]

        try:
            return int(id)
        except (TypeError, ValueError):
            pass

    @classmethod
    def list(cls, options=None):
        """ List hosted certificates. """
        options = options or {}
        return cls.call('cert.hosted.list', options)

    @classmethod
    def info(cls, id):
        """ Display information about a hosted certificate. """
        return cls.call('cert.hosted.info', id)

    @classmethod
    def infos(cls, fqdn):
        """ Display information about hosted certificates for a fqdn. """
        if isinstance(fqdn, (list, tuple)):
            ids = []
            for fqd_ in fqdn:
                ids.extend(cls.infos(fqd_))
            return ids

        ids = cls.usable_id(fqdn)
        if not ids:
            return []

        if not isinstance(ids, (list, tuple)):
            ids = [ids]

        return [cls.info(id_) for id_ in ids]

    @classmethod
    def create(cls, key, crt):
        """ Add a new crt in the hosted cert store. """
        options = {'crt': crt, 'key': key}
        return cls.call('cert.hosted.create', options)

    @classmethod
    def delete(cls, id_):
        """ Remove a cert from the hosted cert store. """
        return cls.call('cert.hosted.delete', id_)

    @classmethod
    def activate_ssl(cls, vhosts, ssl, private_key, poll_cert):
        from .cert import Certificate
        if not ssl:
            return True

        if not isinstance(vhosts, (list, tuple)):
            vhosts = [vhosts]

        missing = []
        for vhost in vhosts:
            try:
                hostedcert = cls.infos(vhost)
                if hostedcert:
                    cls.debug('There is a cert in store for %s' % vhost)
                else:
                    missing.append(vhost)
            except ValueError:
                missing.append(vhost)

        if not missing:
            return True

        vhosts = missing
        cls.debug('Trying to get certificate or generate it for %s' %
                  (', '.join(vhosts)))
        vhost = vhosts[0]
        altnames = vhosts[1:]
        cert = Certificate.get_latest_valid(vhosts)
        if cert:
            if not private_key:
                cls.echo('Please give the private key for certificate id %s '
                         '(CN: %s)' % (cert['id'], cert['cn']))
                return False

            cls.echo('Will use the certificate id %s (CN: %s)' %
                     (cert['id'], cert['cn']))

            if os.path.isfile(private_key):
                with open(private_key) as fhandle:
                    private_key = fhandle.read()

            crt = Certificate.pretty_format_cert(cert)
            cls.create(private_key, crt)
        elif poll_cert:
            cls.echo('This operation will take a long time waiting for the '
                     'certificate to be generated.')

            # create the certificate
            csr = Certificate.process_csr(vhost, private_key=private_key)
            package = Certificate.get_package(vhost, altnames=altnames)
            oper = Certificate.create(csr, 1, package, altnames=altnames)

            cls.echo('If the term close, you can check the certificate create '
                     'operation with :')
            cls.echo('$ gandi certificate follow %s' % oper['id'])
            cls.echo("And when it's DONE you can continue doing :")
            cls.echo('$ gandi certificate export %s' % vhost)
            cls.echo('$ gandi certstore create --private-key %s --certificate '
                     '%s' % (vhost.replace('*.', 'wildcard.') + '.key',
                             vhost.replace('*.', 'wildcard.') + '.crt'))
            cls.echo('And then relaunch the current command.\n')
            cls.echo('Creating the certificate for %s%s' %
                     (vhost,
                      ' (%s)' % ', '.join(altnames) if altnames else ''))
            cls.display_progress(oper)

            # create the hosted certificate.
            # this will always give us a file name.
            _, private_key = Certificate.gen_pk(vhost, private_key)

            with open(private_key) as fhandle:
                private_key = fhandle.read()

            cert = Certificate.get_latest_valid(vhosts)
            crt = Certificate.pretty_format_cert(cert)
            cls.create(private_key, crt)
        else:
            cls.echo('There is no certificate for %s.' % ', '.join(vhosts))
            cls.echo('Create the certificate with (for exemple) :')
            cls.echo('$ gandi certificate create --cn %s --type std %s' %
                     (vhost, '--altnames=%s' % ','.join(altnames) if altnames
                      else ''))
            cls.echo('Or relaunch the current command with --poll-cert option')
        return True



""" Operation namespace commands. """

import click

from gandi.cli.core.cli import cli
from gandi.cli.core.utils import output_generic
from gandi.cli.core.params import pass_gandi, OPER_STEP


@cli.group(name='oper')
@pass_gandi
def oper(gandi):
    """Commands related to Gandi operations."""


@oper.command()
@click.option('--limit', help='Limit number of results.', default=100,
              show_default=True)
@click.option('--step', '-s', type=OPER_STEP, help='Filter the result by step',
              default=['BILL', 'WAIT', 'RUN'], multiple=True,
              show_default=True)
@pass_gandi
def list(gandi, limit, step):
    """List operations."""
    output_keys = ['id', 'type', 'step']

    options = {
        'step': step,
        'items_per_page': limit,
        'sort_by': 'date_created DESC'
    }

    result = gandi.oper.list(options)
    for num, oper in enumerate(reversed(result)):
        if num:
            gandi.separator_line()
        output_generic(gandi, oper, output_keys)

    return result


@oper.command()
@click.argument('id', type=click.INT)
@pass_gandi
def info(gandi, id):
    """Display information about an operation."""
    output_keys = ['id', 'type', 'step', 'last_error']

    oper = gandi.oper.info(id)
    output_generic(gandi, oper, output_keys)

    return oper



""" Iface and vlan commands module. """

from click.exceptions import UsageError

from gandi.cli.core.base import GandiModule
from gandi.cli.modules.datacenter import Datacenter
from gandi.cli.modules.iaas import Iaas


class Ip(GandiModule):
    """ Module to handle CLI commands.

    $ gandi ip list
    $ gandi ip info
    $ gandi ip create
    $ gandi ip attach
    $ gandi ip detach
    $ gandi ip delete

    """

    @classmethod
    def list(cls, options=None):
        """List ip"""
        options = options or {}
        return cls.call('hosting.ip.list', options)

    @classmethod
    def _info(cls, ip_id):
        """ Get information about an ip."""
        return cls.call('hosting.ip.info', ip_id)

    @classmethod
    def info(cls, resource):
        """ Get information about an up."""
        return cls._info(cls.usable_id(resource))

    @classmethod
    def create(cls, ip_version, datacenter, bandwidth, vm=None, vlan=None,
               ip=None, background=False):
        """ Create a public ip and attach it if vm is given. """
        return Iface.create(ip_version, datacenter, bandwidth, vlan, vm, ip,
                            background)

    @classmethod
    def update(cls, resource, params, background=False):
        """ Update this IP """
        cls.echo('Updating your IP')
        result = cls.call('hosting.ip.update', cls.usable_id(resource),
                          params)
        if not background:
            cls.display_progress(result)
        return result

    @classmethod
    def resource_list(cls):
        """ Get the possible list of resources (name, id). """
        items = cls.list({'items_per_page': 500})
        ret = [str(ip['id']) for ip in items]
        ret.extend([ip['ip'] for ip in items])
        return ret

    @classmethod
    def _check_and_detach(cls, ip_, vm_=None):
        # if the ip exists and is attached, we have to detach it
        iface = Iface.info(ip_['iface_id'])

        if iface.get('vm_id'):
            if vm_ and iface['vm_id'] == vm_.get('id'):
                return False

            detach = Iface._detach(iface['id'])
            cls.display_progress(detach)
        return True

    @classmethod
    def attach(cls, ip, vm, background=False, force=False):
        """ Attach """
        vm_ = Iaas.info(vm)
        ip_ = cls.info(ip)
        if not cls._check_and_detach(ip_, vm_):
            return

        # then we should attach the ip to the vm
        attach = Iface._attach(ip_['iface_id'], vm_['id'])
        if not background:
            cls.display_progress(attach)

        return attach

    @classmethod
    def _detach(cls, ip_, iface, background=False, force=False):
        detach = Iface._detach(iface['id'])
        if background:
            return detach

        if detach:
            cls.display_progress(detach)

        return detach

    @classmethod
    def detach(cls, resource, background=False, force=False):
        try:
            ip_ = cls.info(resource)
        except UsageError:
            cls.error("Can't find this ip %s" % resource)

        iface = Iface.info(ip_['iface_id'])
        return cls._detach(ip_, iface, background, force)

    @classmethod
    def delete(cls, resources, background=False, force=False):
        """Delete an ip by deleting the iface"""
        if not isinstance(resources, (list, tuple)):
            resources = [resources]

        ifaces = []
        for item in resources:
            try:
                ip_ = cls.info(item)
            except UsageError:
                cls.error("Can't find this ip %s" % item)

            iface = Iface.info(ip_['iface_id'])
            ifaces.append(iface['id'])

        return Iface.delete(ifaces, background)

    @classmethod
    def from_ip(cls, ip):
        """Retrieve ip id associated to an ip."""
        ips = dict([(ip_['ip'], ip_['id'])
                    for ip_ in cls.list({'items_per_page': 500})])
        return ips.get(ip)

    @classmethod
    def usable_id(cls, id):
        """ Retrieve id from input which can be ip or id."""
        try:
            # id is maybe an ip
            qry_id = cls.from_ip(id)
            if not qry_id:
                qry_id = int(id)
        except Exception:
            qry_id = None

        if not qry_id:
            msg = 'unknown identifier %s' % id
            cls.error(msg)

        return qry_id


class Vlan(GandiModule):

    """ Module to handle CLI commands.

    $ gandi vlan list
    $ gandi vlan info
    $ gandi vlan create
    $ gandi vlan update
    $ gandi vlan delete

    """

    @classmethod
    def list(cls, datacenter=None):
        """List virtual machine vlan

        (in the future it should also handle PaaS vlan)."""
        options = {}
        if datacenter:
            datacenter_id = int(Datacenter.usable_id(datacenter))
            options['datacenter_id'] = datacenter_id

        return cls.call('hosting.vlan.list', options)

    @classmethod
    def resource_list(cls):
        """ Get the possible list of resources (name, id). """
        items = cls.list()
        ret = [vlan['name'] for vlan in items]
        ret.extend([str(vlan['id']) for vlan in items])
        return ret

    @classmethod
    def ifaces(cls, name):
        """ Get vlan attached ifaces. """
        ifaces = Iface.list({'vlan_id': cls.usable_id(name)})
        ret = []
        for iface in ifaces:
            ret.append(Iface.info(iface['id']))
        return ret

    @classmethod
    def _info(cls, vlan_id):
        """ Get information about a vlan."""
        return cls.call('hosting.vlan.info', vlan_id)

    @classmethod
    def info(cls, name):
        """ Get information about a vlan."""
        return cls._info(cls.usable_id(name))

    @classmethod
    def delete(cls, resources, background=False):
        """Delete a vlan."""
        if not isinstance(resources, (list, tuple)):
            resources = [resources]

        opers = []
        for item in resources:
            oper = cls.call('hosting.vlan.delete', cls.usable_id(item))
            if not oper:
                continue

            if isinstance(oper, list):
                opers.extend(oper)
            else:
                opers.append(oper)

        if background:
            return opers

        # interactive mode, run a progress bar
        cls.echo('Deleting your vlan.')
        if opers:
            cls.display_progress(opers)

    @classmethod
    def create(cls, name, datacenter, subnet=None, gateway=None,
               background=False):
        """Create a new vlan."""
        if not background and not cls.intty():
            background = True

        datacenter_id_ = int(Datacenter.usable_id(datacenter))

        vlan_params = {
            'name': name,
            'datacenter_id': datacenter_id_,
        }

        if subnet:
            vlan_params['subnet'] = subnet
        if gateway:
            vlan_params['gateway'] = gateway

        result = cls.call('hosting.vlan.create', vlan_params)

        if not background:
            # interactive mode, run a progress bar
            cls.echo('Creating your vlan.')
            cls.display_progress(result)
            cls.echo('Your vlan %s has been created.' % name)

        return result

    @classmethod
    def update(cls, id, params):
        """Update an existing vlan."""
        cls.echo('Updating your vlan.')
        result = cls.call('hosting.vlan.update', cls.usable_id(id), params)
        return result

    @classmethod
    def from_name(cls, name):
        """Retrieve vlan id associated to a name."""
        result = cls.list()
        vlans = {}
        for vlan in result:
            vlans[vlan['name']] = vlan['id']

        return vlans.get(name)

    @classmethod
    def usable_id(cls, id):
        """ Retrieve id from input which can be name or id."""
        try:
            # id is maybe a name
            qry_id = cls.from_name(id)
            if not qry_id:
                qry_id = int(id)
        except Exception:
            qry_id = None

        if not qry_id:
            msg = 'unknown identifier %s' % id
            cls.error(msg)

        return qry_id


class Iface(GandiModule):

    """ Module to handle CLI commands.

    $ gandi iface list
    $ gandi iface info
    $ gandi iface create
    $ gandi iface delete
    $ gandi iface update

    """

    @classmethod
    def list(cls, options=None):
        """ List all ifaces."""
        options = options or {}
        return cls.call('hosting.iface.list', options)

    @classmethod
    def _info(cls, iface_id):
        """ Get information about an iface."""
        return cls.call('hosting.iface.info', iface_id)

    @classmethod
    def info(cls, num):
        """ Get information about an iface."""
        return cls._info(cls.usable_id(num))

    @classmethod
    def usable_id(cls, id):
        """ Retrieve id from input which can be num or id."""
        try:
            qry_id = int(id)
        except Exception:
            qry_id = None

        if not qry_id:
            msg = 'unknown identifier %s' % id
            cls.error(msg)

        return qry_id

    @classmethod
    def _attach(cls, iface_id, vm_id):
        """ Attach an iface to a vm. """
        oper = cls.call('hosting.vm.iface_attach', vm_id, iface_id)
        return oper

    @classmethod
    def create(cls, ip_version, datacenter, bandwidth, vlan, vm, ip,
               background):
        """ Create a new iface """
        if not background and not cls.intty():
            background = True

        datacenter_id_ = int(Datacenter.usable_id(datacenter))

        iface_params = {
            'ip_version': ip_version,
            'datacenter_id': datacenter_id_,
            'bandwidth': bandwidth,
        }
        if vlan:
            iface_params['vlan'] = Vlan.usable_id(vlan)
            if ip:
                iface_params['ip'] = ip

        result = cls.call('hosting.iface.create', iface_params)

        if background and not vm:
            return result

        # interactive mode, run a progress bar
        cls.echo('Creating your iface.')
        cls.display_progress(result)
        iface_info = cls._info(result['iface_id'])
        cls.echo('Your iface has been created with the following IP '
                 'addresses:')
        for _ip in iface_info['ips']:
            cls.echo('ip%d:\t%s' % (_ip['version'], _ip['ip']))

        if not vm:
            return result

        vm_id = Iaas.usable_id(vm)
        result = cls._attach(result['iface_id'], vm_id)

        if background:
            return result

        cls.echo('Attaching your iface.')
        cls.display_progress(result)
        return result

    @classmethod
    def _detach(cls, iface_id):
        """ Detach an iface from a vm. """
        iface = cls._info(iface_id)
        opers = []
        vm_id = iface.get('vm_id')
        if vm_id:
            cls.echo('The iface is still attached to the vm %s.' % vm_id)
            cls.echo('Will detach it.')
            opers.append(cls.call('hosting.vm.iface_detach', vm_id, iface_id))
        return opers

    @classmethod
    def delete(cls, resources, background=False):
        """ Delete this iface."""
        if not isinstance(resources, (list, tuple)):
            resources = [resources]

        resources = [cls.usable_id(item) for item in resources]

        opers = []
        for iface_id in resources:
            opers.extend(cls._detach(iface_id))

        if opers:
            cls.echo('Detaching your iface(s).')
            cls.display_progress(opers)

        opers = []
        for iface_id in resources:
            oper = cls.call('hosting.iface.delete', iface_id)
            opers.append(oper)

        if background:
            return opers

        cls.echo('Detaching/deleting your iface(s).')
        cls.display_progress(opers)
        return opers

    @classmethod
    def update(cls, id, bandwidth, vm, background):
        """ Update this iface. """
        if not background and not cls.intty():
            background = True

        iface_params = {}
        iface_id = cls.usable_id(id)

        if bandwidth:
            iface_params['bandwidth'] = bandwidth

        if iface_params:
            result = cls.call('hosting.iface.update', iface_id, iface_params)
            if background:
                return result

            # interactive mode, run a progress bar
            cls.echo('Updating your iface %s.' % id)
            cls.display_progress(result)

        if not vm:
            return

        vm_id = Iaas.usable_id(vm)

        opers = cls._detach(iface_id)
        if opers:
            cls.echo('Detaching iface.')
            cls.display_progress(opers)

        result = cls._attach(iface_id, vm_id)

        if background:
            return result

        cls.echo('Attaching your iface.')
        cls.display_progress(result)



""" Domain commands module. """

import time

from gandi.cli.core.base import GandiModule
from gandi.cli.core.utils import DomainNotAvailable


class Domain(GandiModule):

    """ Module to handle CLI commands.

    $ gandi domain create
    $ gandi domain info
    $ gandi domain list

    """

    @classmethod
    def list(cls, options):
        """List operation."""
        return cls.call('domain.list', options)

    @classmethod
    def info(cls, fqdn):
        """Display information about a domain."""
        return cls.call('domain.info', fqdn)

    @classmethod
    def create(cls, fqdn, duration, owner, admin, tech, bill, nameserver,
               extra_parameter, background):
        """Create a domain."""
        fqdn = fqdn.lower()
        if not background and not cls.intty():
            background = True

        result = cls.call('domain.available', [fqdn])
        while result[fqdn] == 'pending':
            time.sleep(1)
            result = cls.call('domain.available', [fqdn])

        if result[fqdn] == 'unavailable':
            raise DomainNotAvailable('%s is not available' % fqdn)

        # retrieve handle of user and save it to configuration
        user_handle = cls.call('contact.info')['handle']
        cls.configure(True, 'api.handle', user_handle)

        owner_ = owner or user_handle
        admin_ = admin or user_handle
        tech_ = tech or user_handle
        bill_ = bill or user_handle

        domain_params = {
            'duration': duration,
            'owner': owner_,
            'admin': admin_,
            'tech': tech_,
            'bill': bill_,
        }

        if nameserver:
            domain_params['nameservers'] = nameserver

        if extra_parameter:
            domain_params['extra'] = {}
            for extra in extra_parameter:
                domain_params['extra'][extra[0]] = extra[1]

        result = cls.call('domain.create', fqdn, domain_params)
        if background:
            return result

        # interactive mode, run a progress bar
        cls.echo('Creating your domain.')
        cls.display_progress(result)
        cls.echo('Your domain %s has been created.' % fqdn)

    @classmethod
    def renew(cls, fqdn, duration, background):
        """Renew a domain."""
        fqdn = fqdn.lower()
        if not background and not cls.intty():
            background = True

        domain_info = cls.info(fqdn)

        current_year = domain_info['date_registry_end'].year
        domain_params = {
            'duration': duration,
            'current_year': current_year,
        }

        result = cls.call('domain.renew', fqdn, domain_params)
        if background:
            return result

        # interactive mode, run a progress bar
        cls.echo('Renewing your domain.')
        cls.display_progress(result)
        cls.echo('Your domain %s has been renewed.' % fqdn)

    @classmethod
    def autorenew_deactivate(cls, fqdn):
        """Activate deautorenew"""
        fqdn = fqdn.lower()

        result = cls.call('domain.autorenew.deactivate', fqdn)

        return result

    @classmethod
    def autorenew_activate(cls, fqdn):
        """Activate autorenew"""
        fqdn = fqdn.lower()

        result = cls.call('domain.autorenew.activate', fqdn)

        return result

    @classmethod
    def from_fqdn(cls, fqdn):
        """Retrieve domain id associated to a FQDN."""
        result = cls.list({'fqdn': fqdn})
        if len(result) > 0:
            return result[0]['id']

    @classmethod
    def usable_id(cls, id):
        """Retrieve id from input which can be fqdn or id."""
        # Check if it's already an integer.
        try:
            qry_id = int(id)
        except Exception:
            # Otherwise, assume it's a FQDN.
            # This will return `None` if the FQDN is not found.
            qry_id = cls.from_fqdn(id)

        if not qry_id:
            msg = 'unknown identifier %s' % id
            cls.error(msg)

        return qry_id



""" Dnssec commands module. """

from gandi.cli.core.base import GandiModule


class DNSSEC(GandiModule):

    """ Module to handle CLI commands.

    $ gandi dnssec create
    $ gandi dnssec list
    $ gandi dnssec delete

    """

    @classmethod
    def list(cls, fqdn):
        """List operation."""
        return cls.call('domain.dnssec.list', fqdn)

    @classmethod
    def create(cls, fqdn, flags, algorithm, public_key):
        """Create a dnssec key."""
        fqdn = fqdn.lower()

        params = {
            'flags': flags,
            'algorithm': algorithm,
            'public_key': public_key,
        }

        result = cls.call('domain.dnssec.create', fqdn, params)

        return result

    @classmethod
    def delete(cls, id):
        """Delete this dnss key."""
        return cls.call('domain.dnssec.delete', id)



# -*- coding: utf-8 -*-

from .base import CommandTestCase
from gandi.cli.commands import root


class RootTestCase(CommandTestCase):

    def test_api(self):

        result = self.invoke_with_exceptions(root.api, [])

        self.assertEqual(result.output, """\
API version: 3.3.42
""")
        self.assertEqual(result.exit_code, 0)



# coding: utf-8

# Author: Rory McCann
# https://pypi.python.org/pypi/ascii_sparks/0.0.3
# License: Unknown

parts = u' ▁▂▃▄▅▆▇▉'


def sparks(nums):
    fraction = max(nums) / float(len(parts) - 1)
    return ''.join([parts[int(round(x / fraction))] for x in nums])



""" Ip namespace commands. """

import click
from click.exceptions import UsageError

from gandi.cli.core.cli import cli
from gandi.cli.core.utils import (
    output_ip, DatacenterLimited
)
from gandi.cli.core.params import (pass_gandi, DATACENTER,
                                   IP_TYPE, option, IntChoice)


@cli.group(name='ip')
@pass_gandi
def ip(gandi):
    """Commands related to hosting IPs."""


@ip.command()
@click.option('--datacenter', type=DATACENTER, default=None,
              help='Filter by datacenter.')
@click.option('--type', default=None, type=IP_TYPE,
              help='Filter by type.')
@click.option('--id', help='Display ids.', is_flag=True)
@click.option('--attached', help='Only display attached ip.', is_flag=True)
@click.option('--detached', help='Only display detached ip.', is_flag=True)
@click.option('--version', help='Display ip version.', is_flag=True)
@click.option('--reverse', help='Display ip reverse.', is_flag=True)
@click.option('--vm', help='Display ip vm.', is_flag=True)
@click.option('--vlan', default=None, help='Filter by vlan.')
@pass_gandi
def list(gandi, datacenter, type, id, attached, detached, version, reverse,
         vm, vlan):
    """List ips."""
    if attached and detached:
        gandi.echo("You can't set --attached and --detached at the same time.")
        return

    output_keys = ['ip', 'state', 'dc', 'type']
    if id:
        output_keys.append('id')
    if version:
        output_keys.append('version')
    if vm:
        output_keys.append('vm')
    if reverse:
        output_keys.append('reverse')

    options = {}
    opt_dc = {}
    if datacenter:
        datacenter_id = int(gandi.datacenter.usable_id(datacenter))
        options['datacenter_id'] = datacenter_id
        opt_dc = {'datacenter_id': datacenter_id}

    iface_options = {}
    if type:
        iface_options['type'] = type
    if vlan:
        iface_options['vlan'] = vlan
    if attached:
        iface_options['state'] = 'used'
    elif detached:
        iface_options['state'] = 'free'

    if iface_options:
        ifaces = gandi.iface.list(iface_options)
        options['iface_id'] = [iface['id'] for iface in ifaces]
    iface_options.update(opt_dc)

    datacenters = gandi.datacenter.list()

    ips = gandi.ip.list(options)
    ifaces = dict([(iface['id'], iface)
                   for iface in gandi.iface.list(iface_options)])
    vms = dict([(vm_['id'], vm_)
                for vm_ in gandi.iaas.list(opt_dc)])

    for num, ip_ in enumerate(ips):
        if num:
            gandi.separator_line()
        output_ip(gandi, ip_, datacenters, vms, ifaces, output_keys)

    return ips


@ip.command()
@click.argument('resource')
@pass_gandi
def info(gandi, resource):
    """Display information about an ip.

    Resource can be an ip or id.
    """
    output_keys = ['ip', 'state', 'dc', 'type', 'vm', 'reverse']

    datacenters = gandi.datacenter.list()

    ip = gandi.ip.info(resource)
    iface = gandi.iface.info(ip['iface_id'])
    vms = None
    if iface.get('vm_id'):
        vm = gandi.iaas.info(iface['vm_id'])
        vms = {vm['id']: vm}

    output_ip(gandi, ip, datacenters, vms, {iface['id']: iface},
              output_keys)

    return ip


@ip.command()
@click.argument('ip')
@click.option('--reverse', help='Update reverse (PTR record) for this IP')
@click.option('--background', '--bg', default=False, is_flag=True,
              help='Run command in background mode (default=False).')
@pass_gandi
def update(gandi, ip, reverse, background):
    """Update an ip."""
    if not reverse:
        return
    return gandi.ip.update(ip, {'reverse': reverse}, background)


@ip.command()
@click.argument('ip')
@click.argument('vm')
@click.option('--background', '--bg', default=False, is_flag=True,
              help='Run command in background mode (default=False).')
@click.option('--force', '-f', is_flag=True,
              help='This is a dangerous option that will cause CLI to continue'
                   ' without prompting. (default=False).')
@pass_gandi
def attach(gandi, ip, vm, background, force):
    """Attach an ip to a vm.

    ip can be an ip id or ip
    vm can be a vm id or name.
    """
    try:
        ip_ = gandi.ip.info(ip)
        vm_ = gandi.iaas.info(vm)
    except UsageError:
        gandi.error("Can't find this ip %s" % ip)

    iface = gandi.iface.info(ip_['iface_id'])
    if iface.get('vm_id'):
        if vm_ and iface['vm_id'] == vm_.get('id'):
            gandi.echo('This ip is already attached to this vm.')
            return

        if not force:
            proceed = click.confirm('Are you sure you want to detach'
                                    ' %s from vm %s' %
                                    (ip_['ip'], iface['vm_id']))
            if not proceed:
                return

    return gandi.ip.attach(ip, vm, background, force)


@ip.command()
@click.option('--datacenter', type=DATACENTER,
              help='Datacenter where the ip will be created.')
@option('--bandwidth', type=click.INT, default=102400,
        help="Network bandwidth in kbit/s used to create the VM's first "
             "network interface.")
@option('--ip-version', type=IntChoice(['4', '6']), default=4,
        help='Version of created IP.')
@click.option('--vlan', help='The vlan to which attach this ip if any.')
@click.option('--ip', help='The ip if you try to create a private ip.')
@click.option('--attach', help='The vm you want to attach if any.')
@click.option('--background', '--bg', default=False, is_flag=True,
              help='Run command in background mode (default=False).')
@pass_gandi
def create(gandi, datacenter, bandwidth, ip_version, vlan, ip, attach,
           background):
    """Create a public or private ip
    """
    if ip_version != 4 and vlan:
        gandi.echo('You must have an --ip-version to 4 when having a vlan.')
        return

    if ip and not vlan:
        gandi.echo('You must have a --vlan when giving an --ip.')
        return

    vm_ = gandi.iaas.info(attach) if attach else None

    if datacenter and vm_:
        dc_id = gandi.datacenter.usable_id(datacenter)
        if dc_id != vm_['datacenter_id']:
            gandi.echo('The datacenter you provided does not match the '
                       'datacenter of the vm you want to attach to.')
            return

    if not datacenter:
        datacenter = vm_['datacenter_id'] if vm_ else 'LU'

    try:
        gandi.datacenter.is_opened(datacenter, 'iaas')
    except DatacenterLimited as exc:
        gandi.echo('/!\ Datacenter %s will be closed on %s, '
                   'please consider using another datacenter.' %
                   (datacenter, exc.date))

    return gandi.ip.create(ip_version, datacenter, bandwidth, attach,
                           vlan, ip, background)


@ip.command()
@click.argument('resource')
@click.option('--background', '--bg', default=False, is_flag=True,
              help='Run command in background mode (default=False).')
@click.option('--force', '-f', is_flag=True,
              help='This is a dangerous option that will cause CLI to continue'
                   ' without prompting. (default=False).')
@pass_gandi
def detach(gandi, resource, background, force):
    """Detach an ip from it's currently attached vm.

    resource can be an ip id or ip.
    """
    if not force:
        proceed = click.confirm('Are you sure you want to detach ip %s?' %
                                resource)
        if not proceed:
            return

    return gandi.ip.detach(resource, background, force)


@ip.command()
@click.argument('resource', nargs=-1, required=True)
@click.option('--background', '--bg', default=False, is_flag=True,
              help='Run command in background mode (default=False).')
@click.option('--force', '-f', is_flag=True,
              help='This is a dangerous option that will cause CLI to continue'
                   ' without prompting. (default=False).')
@pass_gandi
def delete(gandi, resource, background, force):
    """Delete one or more IPs (after detaching them from VMs if necessary).

    resource can be an ip id or ip.
    """
    resource = sorted(tuple(set(resource)))
    possible_resources = gandi.ip.resource_list()

    # check that each IP can be deleted
    for item in resource:
        if item not in possible_resources:
            gandi.echo('Sorry interface %s does not exist' % item)
            gandi.echo('Please use one of the following: %s' %
                       possible_resources)
            return

    if not force:
        proceed = click.confirm('Are you sure you want to delete ip(s) %s' %
                                ', '.join(resource))
        if not proceed:
            return

    return gandi.ip.delete(resource, background, force)



# -*- coding: utf-8 -*-

from ..compat import mock
from .base import CommandTestCase
from gandi.cli.commands import certstore


class CertStoreTestCase(CommandTestCase):

    def test_list(self):

        result = self.invoke_with_exceptions(certstore.list, [])

        self.assertEqual(result.output, """\
subject   : /OU=Domain Control Validated/OU=Gandi Standard SSL/CN=test1.domain.fr
----------
subject   : /OU=Domain Control Validated/OU=Gandi Standard SSL/CN=test1.domain.fr
----------
subject   : /OU=Domain Control Validated/OU=Gandi Standard SSL/CN=test2.domain.fr
----------
subject   : /OU=Domain Control Validated/OU=Gandi Standard SSL/CN=test3.domain.fr
----------
subject   : /OU=Domain Control Validated/OU=Gandi Standard SSL/CN=test4.domain.fr
----------
subject   : /OU=Domain Control Validated/OU=Gandi Standard Wildcard SSL/CN=*.domain.fr
""")  # noqa
        self.assertEqual(result.exit_code, 0)

    def test_list_all(self):

        result = self.invoke_with_exceptions(certstore.list, ['--id',
                                                              '--vhosts',
                                                              '--dates',
                                                              '--fqdns'])
        self.assertEqual(result.output, """\
id          : 1
subject     : /OU=Domain Control Validated/OU=Gandi Standard SSL/CN=test1.domain.fr
date_created: 20150407T00:00:00
date_expire : 20160316T00:00:00
\t----------
\tfqdn      : test1.domain.fr
\t----------
\tvhost     : test1.domain.fr
\ttype      : paas
----------
id          : 2
subject     : /OU=Domain Control Validated/OU=Gandi Standard SSL/CN=test1.domain.fr
date_created: 20150407T00:00:00
date_expire : 20160316T00:00:00
\t----------
\tfqdn      : test1.domain.fr
\t----------
\tvhost     : test1.domain.fr
\ttype      : paas
----------
id          : 3
subject     : /OU=Domain Control Validated/OU=Gandi Standard SSL/CN=test2.domain.fr
date_created: 20150408T00:00:00
date_expire : 20160408T00:00:00
\t----------
\tfqdn      : test2.domain.fr
----------
id          : 4
subject     : /OU=Domain Control Validated/OU=Gandi Standard SSL/CN=test3.domain.fr
date_created: 20150408T00:00:00
date_expire : 20160408T00:00:00
\t----------
\tfqdn      : test3.domain.fr
----------
id          : 5
subject     : /OU=Domain Control Validated/OU=Gandi Standard SSL/CN=test4.domain.fr
date_created: 20150408T00:00:00
date_expire : 20160408T00:00:00
\t----------
\tfqdn      : test4.domain.fr
----------
id          : 6
subject     : /OU=Domain Control Validated/OU=Gandi Standard Wildcard SSL/CN=*.domain.fr
date_created: 20150409T00:00:00
date_expire : 20160409T00:00:00
\t----------
\tfqdn      : *.domain.fr
\t----------
\tvhost     : *.domain.fr
\ttype      : paas
""")  # noqa
        self.assertEqual(result.exit_code, 0)

    def test_info_fqdn(self):

        result = self.invoke_with_exceptions(
            certstore.info, ['test1.domain.fr'])

        self.assertEqual(result.output, """\
id          : 1
subject     : /OU=Domain Control Validated/OU=Gandi Standard SSL/CN=test1.domain.fr
date_created: 20150407T00:00:00
date_expire : 20160316T00:00:00
\t----------
\tfqdn      : test1.domain.fr
\t----------
\tvhost     : test1.domain.fr
\ttype      : paas
----------
id          : 2
subject     : /OU=Domain Control Validated/OU=Gandi Standard SSL/CN=test1.domain.fr
date_created: 20150407T00:00:00
date_expire : 20160316T00:00:00
\t----------
\tfqdn      : test1.domain.fr
\t----------
\tvhost     : test1.domain.fr
\ttype      : paas
""")  # noqa
        self.assertEqual(result.exit_code, 0)

    def test_info_id(self):
        result = self.invoke_with_exceptions(certstore.info, ['1'])

        self.assertEqual(result.output, """\
id          : 1
subject     : /OU=Domain Control Validated/OU=Gandi Standard SSL/CN=test1.domain.fr
date_created: 20150407T00:00:00
date_expire : 20160316T00:00:00
\t----------
\tfqdn      : test1.domain.fr
\t----------
\tvhost     : test1.domain.fr
\ttype      : paas
""")  # noqa
        self.assertEqual(result.exit_code, 0)

    def test_create(self):
        result = self.invoke_with_exceptions(certstore.create,
                                             ['--pk', 'PK', '--crt', 'CRT'])

        self.assertEqual(result.output, """\
id          : 5
subject     : /OU=Domain Control Validated/OU=Gandi Standard SSL/CN=test4.domain.fr
date_created: 20150408T00:00:00
date_expire : 20160408T00:00:00
\t----------
\tfqdn      : test4.domain.fr
""")  # noqa
        self.assertEqual(result.exit_code, 0)

    def test_create_id(self):
        result = self.invoke_with_exceptions(certstore.create,
                                             ['--pk', 'PK', '--crt-id', '701'])

        self.assertEqual(result.output, """\
id          : 5
subject     : /OU=Domain Control Validated/OU=Gandi Standard SSL/CN=test4.domain.fr
date_created: 20150408T00:00:00
date_expire : 20160408T00:00:00
\t----------
\tfqdn      : test4.domain.fr
""")  # noqa
        self.assertEqual(result.exit_code, 0)

    def test_create_missing(self):
        result = self.invoke_with_exceptions(certstore.create,
                                             ['--pk', 'PK'])

        self.assertEqual(result.output, """\
One of --certificate or --certificate-id is needed.
""")
        self.assertEqual(result.exit_code, 0)

    def test_create_too_many(self):
        args = ['--pk', 'PK', '--crt', 'CRT', '--crt-id', '999']
        result = self.invoke_with_exceptions(certstore.create, args)

        self.assertEqual(result.output, """\
Only one of --certificate or --certificate-id is needed.
id          : 5
subject     : /OU=Domain Control Validated/OU=Gandi Standard SSL/CN=test4.domain.fr
date_created: 20150408T00:00:00
date_expire : 20160408T00:00:00
\t----------
\tfqdn      : test4.domain.fr
""")  # noqa
        self.assertEqual(result.exit_code, 0)

    def test_create_parameter_files(self):
        args = ['--pk', '/tmp/pk.key', '--crt', '/tmp/key.crt']
        with mock.patch('gandi.cli.commands.certstore.os.path.isfile',
                        create=True) as mock_isfile:
            mock_isfile.return_value = True

            with mock.patch('gandi.cli.commands.certstore.open',
                            create=True) as mock_open:
                mock_open.return_value = mock.MagicMock()

                result = self.invoke_with_exceptions(certstore.create, args)

        self.assertEqual(result.output, """\
id          : 5
subject     : /OU=Domain Control Validated/OU=Gandi Standard SSL/CN=test4.domain.fr
date_created: 20150408T00:00:00
date_expire : 20160408T00:00:00
\t----------
\tfqdn      : test4.domain.fr
""")  # noqa
        self.assertEqual(result.exit_code, 0)

    def test_delete(self):
        result = self.invoke_with_exceptions(certstore.delete, ['1'])

        self.assertEqual(result.output, """\
Are you sure to delete the following hosted certificates ?
1: /OU=Domain Control Validated/OU=Gandi Standard SSL/CN=test1.domain.fr
 [y/N]: \

""")
        self.assertEqual(result.exit_code, 0)

        result = self.invoke_with_exceptions(certstore.delete, ['1', '-f'])
        self.assertEqual(result.output, """\
""")
        self.assertEqual(result.exit_code, 0)

    def test_delete_unknown(self):
        args = ['100.fr', '-f']
        result = self.invoke_with_exceptions(certstore.delete, args)

        self.assertEqual(result.output, '')
        self.assertEqual(result.exit_code, 0)



"""
Security enhancements for xmlrpc.
"""
from __future__ import print_function

import sys

try:
    import xmlrpc.client as xmlrpclib
except ImportError:
    import xmlrpclib

try:
    import requests
except ImportError:
    print('python requests is required, please reinstall.', file=sys.stderr)
    sys.exit(1)


class RequestsTransport(xmlrpclib.Transport):
    """
    Drop in Transport for xmlrpclib that uses Requests instead of httplib
    # https://gist.github.com/chrisguitarguy/2354951
    # https://github.com/mardiros/pyshop/blob/master/pyshop/helpers/pypi.py
    """

    use_https = True

    def __init__(self, use_datetime=0, host=None):
        xmlrpclib.Transport.__init__(self, use_datetime)
        if host:
            self.use_https = 'https' in host

    def request(self, host, handler, request_body, verbose):
        """
        Make an xmlrpc request.
        """
        headers = {'User-Agent': self.user_agent,
                   'Accept': 'text/xml',
                   'Content-Type': 'text/xml'}

        url = self._build_url(host, handler)
        try:
            resp = requests.post(url, data=request_body, headers=headers)
        except ValueError:
            raise
        except Exception:
            raise  # something went wrong
        else:
            try:
                resp.raise_for_status()
            except requests.RequestException as e:
                raise xmlrpclib.ProtocolError(url, resp.status_code,
                                              str(e), resp.headers)
            else:
                return self.parse_response(resp)

    def parse_response(self, resp):
        """
        Parse the xmlrpc response.
        """
        p, u = self.getparser()
        p.feed(resp.content)
        p.close()
        return u.close()

    def _build_url(self, host, handler):
        """
        Build a url for our request based on the host, handler and use_https
        property
        """
        scheme = 'https' if self.use_https else 'http'
        return '%s://%s%s' % (scheme, host, handler)



""" Mail namespace commands. """

import click

from gandi.cli.core.cli import cli
from gandi.cli.core.utils import output_mailbox, output_list
from gandi.cli.core.params import pass_gandi, EMAIL_TYPE


@cli.group(name='mail')
@pass_gandi
def mail(gandi):
    """Commands related to domain mailboxes."""


@mail.command()
@click.option('--limit', help='Limit number of results.',
              default=100, show_default=True)
@click.argument('domain', metavar='domain.tld')
@pass_gandi
def list(gandi, domain, limit):
    """List mailboxes created on a domain."""
    options = {'items_per_page': limit}
    mailboxes = gandi.mail.list(domain, options)
    output_list(gandi, [mbox['login'] for mbox in mailboxes])
    return mailboxes


@mail.command()
@click.argument('email', type=EMAIL_TYPE, metavar='login@domain.tld')
@pass_gandi
def info(gandi, email):
    """Display information about a mailbox."""
    login, domain = email

    output_keys = ['login', 'aliases', 'fallback', 'quota', 'responder']
    mailbox = gandi.mail.info(domain, login)
    output_mailbox(gandi, mailbox, output_keys)

    return mailbox


@mail.command()
@click.option('--quota', '-q', help='Set quota on mailbox. 0 is unlimited.',
              default=None, type=click.INT)
@click.option('--fallback', '-f', help='Add fallback address.',
              default=None)
@click.option('--alias', '-a', help='Add mailbox alias.',
              multiple=True, required=False)
@click.option('--password', '-p', default=None, type=click.STRING,
              required=False, help='Prompt a password to create a mailbox.')
@click.argument('email', type=EMAIL_TYPE, metavar='login@domain.tld')
@pass_gandi
def create(gandi, email, quota, fallback, alias, password):
    """Create a mailbox."""
    login, domain = email
    options = {}

    if not password:
        password = click.prompt('password', hide_input=True,
                                confirmation_prompt=True)
    options['password'] = password
    if quota is not None:
        options['quota'] = quota
    if fallback is not None:
        options['fallback_email'] = fallback

    result = gandi.mail.create(domain, login, options, alias)

    return result


@mail.command()
@click.option('--force', '-f', is_flag=True,
              help='This is a dangerous option that will cause CLI to continue'
                   ' without prompting. (default=False).')
@click.argument('email', type=EMAIL_TYPE, metavar='login@domain.tld')
@pass_gandi
def delete(gandi, email, force):
    """Delete a mailbox."""
    login, domain = email

    if not force:
        proceed = click.confirm('Are you sure to delete the '
                                'mailbox %s@%s ?' % (login, domain))

        if not proceed:
            return

    result = gandi.mail.delete(domain, login)

    return result


@mail.command()
@click.option('--password', '-p', help='Prompt a password to set a mailbox.',
              is_flag=True)
@click.option('--quota', '-q', help='Set quota on mailbox. 0 is unlimited.',
              default=None, type=click.INT)
@click.option('--fallback', '-f', help='Add fallback address.',
              default=None, show_default=True)
@click.option('--alias-add', '-a', help='Add mailbox alias.',
              multiple=True, required=False)
@click.option('--alias-del', '-d', help='Delete mailbox alias.',
              multiple=True, required=False)
@click.argument('email', type=EMAIL_TYPE, metavar='login@domain.tld')
@pass_gandi
def update(gandi, email, password, quota, fallback, alias_add, alias_del):
    """Update a mailbox."""
    options = {}

    if password:
        password = click.prompt('password', hide_input=True,
                                confirmation_prompt=True)
        options['password'] = password

    if quota is not None:
        options['quota'] = quota

    if fallback is not None:
        options['fallback_email'] = fallback

    login, domain = email

    result = gandi.mail.update(domain, login, options, alias_add, alias_del)

    return result


@mail.command()
@click.option('--background', '--bg', default=False, is_flag=True,
              help='Run command in background mode (default=False).')
@click.option('--force', '-f', is_flag=True,
              help='This is a dangerous option that will cause CLI to continue'
                   ' without prompting. (default=False).')
@click.option('--alias', '-a', help='Purge all aliases.', default=False,
              is_flag=True)
@click.argument('email', type=EMAIL_TYPE, metavar='login@domain.tld')
@pass_gandi
def purge(gandi, email, background, force, alias):
    """Purge a mailbox."""
    login, domain = email
    if alias:
        if not force:
            proceed = click.confirm('Are you sure to purge all aliases for '
                                    'mailbox %s@%s ?' % (login, domain))
            if not proceed:
                return
        result = gandi.mail.set_alias(domain, login, [])
    else:
        if not force:
            proceed = click.confirm('Are you sure to purge mailbox %s@%s ?'
                                    % (login, domain))
            if not proceed:
                return
        result = gandi.mail.purge(domain, login, background)

    return result



# -*- coding: utf-8 -*-
import sys

from .base import CommandTestCase
from gandi.cli.commands import dnssec


class DnssecTestCase(CommandTestCase):

    def test_list(self):

        result = self.invoke_with_exceptions(dnssec.list, ['iheartcli.com'])

        self.maxDiff = None
        if sys.version_info[0] == 2:
            self.assertEqual(result.output, """[{'algorithm': 5,\n  'date_created': datetime.datetime(2012, 2, 24, 17, 16, 8),\n  'digest': '457c626c008cc70d68133254abc4ee4eb79e4e6c99f9423b60b543fa8a69e6ac',\n  'digest_type': 2,\n  'flags': 257,\n  'id': 125,\n  'keytag': 9301,\n  'public_key': 'AwEAAdYixYvq9eJLRQcxUeYJWaxAGXiP/K1/C7XHbUWGzA8AHCRp81FAmfwcw1FrJ7bMViEegewPDGciQSv5HotPPOynUmkZbgztOeejH/+3Il/cM8SW4Et0i+99S7l9as+FI3AYOhsllDJK1WM9smn0S/9igfpR2dGmCyDU ZfeR1A49\\n'}]\n""")  # noqa
        self.assertEqual(result.exit_code, 0)

    def test_create(self):
        result = self.invoke_with_exceptions(
            dnssec.create,
            ['iheartcli.com',
             '--flags', '257',
             '--algorithm', '5',
             '--public_key', 'AwEAAdYixYvq9eJLRQcxUeYJWaxAGXiP/K1/C7XHbUWGzA8A'
                             'HCRp81FAmfwcw1FrJ7bMViEegewPDGciQSv5HotPPOynUmkZ'
                             'bgztOeejH/+3Il/cM8SW4Et0i+99S7l9as+FI3AYOhsllDJK'
                             '1WM9smn0S/9igfpR2dGmCyDU ZfeR1A49',
             ])

        self.assertEqual(result.output, """""")
        self.assertEqual(result.exit_code, 0)

    def test_delete(self):

        result = self.invoke_with_exceptions(dnssec.delete, ['125'])

        self.assertEqual(result.output, """Delete successful.\n""")
        self.assertEqual(result.exit_code, 0)

    def test_help(self):

        result = self.invoke_with_exceptions(dnssec.create, ['--help'])

        self.assertIn('--algorithm [1|2|3|5|6|7|8|10|12|13|14|15|16|253|254]', result.output)
        self.assertIn('--flags [256|257]', result.output)

        self.assertEqual(result.exit_code, 0)



""" Forward commands module. """

from gandi.cli.core.base import GandiModule


class Forward(GandiModule):

    """ Module to handle CLI commands.

    $ gandi forward create
    $ gandi forward delete
    $ gandi forward list
    $ gandi forward update

    """

    @classmethod
    def list(cls, domain, options):
        """List forwards for a given domain name."""
        return cls.call('domain.forward.list', domain, options)

    @classmethod
    def delete(cls, domain, source):
        """Delete a domain mail forward."""
        return cls.call('domain.forward.delete', domain, source)

    @classmethod
    def create(cls, domain, source, destinations):
        """Create a domain mail forward."""
        cls.echo('Creating mail forward %s@%s' % (source, domain))
        options = {'destinations': list(destinations)}
        result = cls.call('domain.forward.create', domain, source, options)

        return result

    @classmethod
    def get_destinations(cls, domain, source):
        """Retrieve forward information."""
        forwards = cls.list(domain, {'items_per_page': 500})
        for fwd in forwards:
            if fwd['source'] == source:
                return fwd['destinations']

        return []

    @classmethod
    def update(cls, domain, source, dest_add, dest_del):
        """Update a domain mail forward destinations."""
        result = None

        if dest_add or dest_del:
            current_destinations = cls.get_destinations(domain, source)
            fwds = current_destinations[:]
            if dest_add:
                for dest in dest_add:
                    if dest not in fwds:
                        fwds.append(dest)
            if dest_del:
                for dest in dest_del:
                    if dest in fwds:
                        fwds.remove(dest)

            if ((len(current_destinations) != len(fwds))
                    or (current_destinations != fwds)):
                cls.echo('Updating mail forward %s@%s' % (source, domain))
                options = {'destinations': fwds}
                result = cls.call('domain.forward.update', domain, source,
                                  options)

        return result



""" SSH keys namespace commands. """

import click
from click.exceptions import UsageError

from gandi.cli.core.cli import cli
from gandi.cli.core.utils import output_sshkey
from gandi.cli.core.params import pass_gandi


@cli.group(name='sshkey')
@pass_gandi
def sshkey(gandi):
    """Commands related to hosting sshkeys."""


@sshkey.command()
@click.option('--id', help='Display ids.', is_flag=True)
@click.option('--limit', help='Limit number of results.', default=100,
              show_default=True)
@pass_gandi
def list(gandi, id, limit):
    """ List SSH keys. """
    options = {
        'items_per_page': limit,
    }

    output_keys = ['name', 'fingerprint']

    if id:
        output_keys.append('id')

    result = gandi.sshkey.list(options)
    for num, sshkey in enumerate(result):
        if num:
            gandi.separator_line()
        output_sshkey(gandi, sshkey, output_keys)

    return result


@sshkey.command()
@click.argument('resource', nargs=-1, required=True)
@click.option('--id', help='Display ids.', is_flag=True)
@click.option('--value', help='Display value.', is_flag=True)
@pass_gandi
def info(gandi, resource, id, value):
    """Display information about an SSH key.

    Resource can be a name or an ID
    """
    output_keys = ['name', 'fingerprint']
    if id:
        output_keys.append('id')

    if value:
        output_keys.append('value')

    ret = []
    for item in resource:
        sshkey = gandi.sshkey.info(item)
        ret.append(output_sshkey(gandi, sshkey, output_keys))

    return ret


@sshkey.command()
@click.option('--name', help='SSH key name.', required=True)
@click.option('--value', help='Content of the SSH key.')
@click.option('--filename', type=click.File('r'), help='SSH key file.')
@pass_gandi
def create(gandi, name, value=None, filename=None):
    """ Create a new SSH key. """
    if not value and not filename:
        raise UsageError('You must set value OR filename.')

    if value and filename:
        raise UsageError('You must not set value AND filename.')

    if filename:
        value = filename.read()

    ret = gandi.sshkey.create(name, value)

    output_keys = ['id', 'name', 'fingerprint']
    return output_sshkey(gandi, ret, output_keys)


@sshkey.command()
@click.argument('resource', nargs=-1, required=True)
@pass_gandi
def delete(gandi, resource):
    """Delete SSH keys.

    Resource can be a name or an ID
    """
    for item in resource:
        gandi.sshkey.delete(item)



""" DNS namespace commands. """

import sys
import click

from gandi.cli.core.cli import cli
from gandi.cli.core.utils import output_dns_records, output_generic
from gandi.cli.core.params import pass_gandi, DNS_RECORDS


@cli.group(name='dns')
@pass_gandi
def dns(gandi):
    """Commands related to LiveDNS."""


@dns.command('domain.list')
@pass_gandi
def domain_list(gandi):
    """List domains manageable by REST API."""
    domains = gandi.dns.list()
    for domain in domains:
        gandi.echo(domain['fqdn'])

    return domains


@dns.command()
@click.argument('fqdn')
@click.argument('name', default=None, required=False)
@click.argument('rrset_type', default=None, required=False, type=DNS_RECORDS)
@click.option('--sort', default='name',
              type=click.Choice(['name', 'ttl', 'type', 'values']),
              help='Sort results (does not work with text option).')
@click.option('--type', default=None, type=DNS_RECORDS,
              help='Filter results by type (does not work with text option).')
@click.option('--text', default=False, is_flag=True,
              help='Output result as text.')
@pass_gandi
def list(gandi, fqdn, name, sort, type, rrset_type, text):
    """Display records for a domain."""
    domains = gandi.dns.list()
    domains = [domain['fqdn'] for domain in domains]
    if fqdn not in domains:
        gandi.echo('Sorry domain %s does not exist' % fqdn)
        gandi.echo('Please use one of the following: %s' % ', '.join(domains))
        return

    output_keys = ['name', 'ttl', 'type', 'values']

    result = gandi.dns.records(fqdn, sort_by=sort, text=text)
    if text:
        gandi.echo(result)
        return result

    for num, rec in enumerate(result):
        if type and rec['rrset_type'] != type:
            continue
        if rrset_type and rec['rrset_type'] != rrset_type:
            continue
        if name and rec['rrset_name'] != name:
            continue
        if num:
            gandi.separator_line()
        output_dns_records(gandi, rec, output_keys)

    return result


@dns.command()
@click.argument('fqdn')
@click.argument('name')
@click.argument('type', type=DNS_RECORDS)
@click.argument('value', nargs=-1, required=True)
@click.option('--ttl', default=None, type=click.INT, required=False,
              help='Time to live, in seconds')
@pass_gandi
def create(gandi, fqdn, name, type, value, ttl):
    """Create new record entry for a domain.

    multiple value parameters can be provided.
    """
    domains = gandi.dns.list()
    domains = [domain['fqdn'] for domain in domains]
    if fqdn not in domains:
        gandi.echo('Sorry domain %s does not exist' % fqdn)
        gandi.echo('Please use one of the following: %s' % ', '.join(domains))
        return

    result = gandi.dns.add_record(fqdn, name, type, value, ttl)
    gandi.echo(result['message'])


@dns.command()
@click.argument('fqdn')
@click.argument('name', required=False)
@click.argument('type', type=DNS_RECORDS, required=False)
@click.argument('value', nargs=-1, required=False)
@click.option('--ttl', default=None, type=click.INT, required=False,
              help='Time to live, in seconds')
@click.option('-f', '--file', type=click.File('rb'),
              help='Zone content in a plain text file. If provided this will '
                   'ignore all other parameters and overwrite current zone '
                   'content')
@pass_gandi
def update(gandi, fqdn, name, type, value, ttl, file):
    """Update record entry for a domain.

    --file option will ignore other parameters and overwrite current zone
    content with provided file content.
    """
    domains = gandi.dns.list()
    domains = [domain['fqdn'] for domain in domains]
    if fqdn not in domains:
        gandi.echo('Sorry domain %s does not exist' % fqdn)
        gandi.echo('Please use one of the following: %s' % ', '.join(domains))
        return

    content = ''
    if file:
        content = file.read()
    elif not sys.stdin.isatty():
        content = click.get_text_stream('stdin').read()

    content = content.strip()
    if not content and not name and not type and not value:
        click.echo('Cannot find parameters for zone content to update.')
        return

    if name and type and not value:
        click.echo('You must provide one or more value parameter.')
        return

    result = gandi.dns.update_record(fqdn, name, type, value, ttl, content)
    gandi.echo(result['message'])


@dns.command()
@click.argument('fqdn')
@click.argument('name', required=False)
@click.argument('type', type=DNS_RECORDS, required=False)
@click.option('--force', '-f', is_flag=True,
              help='This is a dangerous option that will cause CLI to continue'
                   ' without prompting. (default=False).')
@pass_gandi
def delete(gandi, fqdn, name, type, force):
    """Delete record entry for a domain."""
    domains = gandi.dns.list()
    domains = [domain['fqdn'] for domain in domains]
    if fqdn not in domains:
        gandi.echo('Sorry domain %s does not exist' % fqdn)
        gandi.echo('Please use one of the following: %s' % ', '.join(domains))
        return

    if not force:
        if not name and not type:
            prompt = ("Are you sure to delete all records for domain %s ?" %
                      fqdn)
        elif name and not type:
            prompt = ("Are you sure to delete all '%s' name records for "
                      "domain %s ?" % (name, fqdn))
        else:
            prompt = ("Are you sure to delete all '%s' records of type %s "
                      "for domain %s ?" % (name, type, fqdn))

        proceed = click.confirm(prompt)

        if not proceed:
            return

    result = gandi.dns.del_record(fqdn, name, type)
    gandi.echo('Delete successful.')
    return result


@dns.group(name='keys')
@pass_gandi
def keys(gandi):
    """Commands related to LiveDNS DNSSEC keys."""


@keys.command(name='list')
@click.argument('fqdn')
@pass_gandi
def keys_list(gandi, fqdn):
    """List domain keys."""
    keys = gandi.dns.keys(fqdn)
    output_keys = ['uuid', 'algorithm', 'algorithm_name', 'ds', 'flags',
                   'status']
    for num, key in enumerate(keys):
        if num:
            gandi.separator_line()
        output_generic(gandi, key, output_keys, justify=15)
    return keys


@keys.command(name='info')
@click.argument('fqdn')
@click.argument('key')
@pass_gandi
def keys_info(gandi, fqdn, key):
    """Display information about a domain key."""
    key_info = gandi.dns.keys_info(fqdn, key)
    output_keys = ['uuid', 'algorithm', 'algorithm_name', 'ds', 'fingerprint',
                   'public_key', 'flags', 'tag', 'status']
    output_generic(gandi, key_info, output_keys, justify=15)
    return key_info


@keys.command(name='create')
@click.argument('fqdn')
@click.argument('flag', type=click.Choice(['256', '257']))
@pass_gandi
def keys_create(gandi, fqdn, flag):
    """Create key for a domain."""
    key_info = gandi.dns.keys_create(fqdn, int(flag))

    output_keys = ['uuid', 'algorithm', 'algorithm_name', 'ds', 'fingerprint',
                   'public_key', 'flags', 'tag', 'status']
    output_generic(gandi, key_info, output_keys, justify=15)
    return key_info


@keys.command(name='delete')
@click.argument('fqdn')
@click.argument('key')
@click.option('--force', '-f', is_flag=True,
              help='This is a dangerous option that will cause CLI to continue'
                   ' without prompting. (default=False).')
@pass_gandi
def keys_delete(gandi, fqdn, key, force):
    """Delete a key for a domain."""
    if not force:
        proceed = click.confirm('Are you sure you want to delete key %s on '
                                'domain %s?' % (key, fqdn))

        if not proceed:
            return

    result = gandi.dns.keys_delete(fqdn, key)
    gandi.echo('Delete successful.')
    return result


@keys.command(name='recover')
@click.argument('fqdn')
@click.argument('key')
@pass_gandi
def keys_recover(gandi, fqdn, key):
    """Recover deleted key for a domain."""
    result = gandi.dns.keys_recover(fqdn, key)
    gandi.echo('Recover successful.')
    return result



try:
    # python3
    from xmlrpc.client import DateTime
except ImportError:
    # python2
    from xmlrpclib import DateTime


def snapshotprofile_list(options):
    ret = [{'id': 7,
            'kept_total': 3,
            'name': 'paas_normal',
            'quota_factor': 1.3,
            'schedules': [{'kept_version': 1, 'name': 'daily'},
                          {'kept_version': 1, 'name': 'weekly'},
                          {'kept_version': 1, 'name': 'weekly4'}]}]

    for fkey in options:
        ret = [snp for snp in ret if snp[fkey] == options[fkey]]

    return ret


def list(options):

    ret = [{'catalog_name': 'phpmysql_s',
            'console': '1656411@console.dc0.gpaas.net',
            'data_disk_additional_size': 0,
            'datacenter_id': 1,
            'date_end': DateTime('20160408T00:00:00'),
            'date_end_commitment': None,
            'date_start': DateTime('20130903T22:14:13'),
            'id': 126276,
            'name': 'paas_owncloud',
            'need_upgrade': False,
            'servers': [{'id': 126273}],
            'size': 's',
            'snapshot_profile': None,
            'state': 'halted',
            'type': 'phpmysql'},
           {'catalog_name': 'nodejsmongodb_s',
            'console': '185290@console.dc2.gpaas.net',
            'data_disk_additional_size': 0,
            'datacenter_id': 3,
            'date_end': DateTime('20161125T15:52:56'),
            'date_end_commitment': DateTime('20151118T18:00:00'),
            'date_start': DateTime('20141025T15:52:56'),
            'id': 163744,
            'name': 'paas_cozycloud',
            'need_upgrade': False,
            'servers': [{'id': 163728}],
            'size': 's',
            'snapshot_profile': None,
            'state': 'running',
            'type': 'nodejsmongodb'}]

    options.pop('items_per_page', None)

    for fkey in options:
        ret = [paas for paas in ret if paas[fkey] == options[fkey]]

    return ret


def info(paas_id):

    ret = [{'autorenew': None,
            'catalog_name': 'nodejsmongodb_s',
            'console': '185290@console.dc2.gpaas.net',
            'data_disk_additional_size': 0,
            'datacenter': {'country': 'Luxembourg',
                           'dc_code': 'LU-BI1',
                           'id': 3,
                           'iso': 'LU',
                           'name': 'Bissen'},
            'datadisk_total_size': 10.0,
            'date_end': DateTime('20161125T15:52:56'),
            'date_end_commitment': DateTime('20151118T00:00:00'),
            'date_start': DateTime('20141025T15:52:56'),
            'ftp_server': 'sftp.dc2.gpaas.net',
            'git_server': 'git.dc2.gpaas.net',
            'id': 163744,
            'name': 'paas_cozycloud',
            'need_upgrade': False,
            'owner': {'handle': 'PXP561-GANDI', 'id': 2920674},
            'servers': [{'graph_urls': {'vcpu': [''], 'vdi': [''],
                                        'vif': ['']},
                         'id': 163728,
                         'uuid': 19254}],
            'size': 's',
            'snapshot_profile': None,
            'state': 'running',
            'type': 'nodejsmongodb',
            'user': 185290,
            'vhosts': [{'date_creation': DateTime('20141025T15:50:54'),
                        'id': 1177216,
                        'name': '187832c2b34.testurl.ws',
                        'state': 'running'},
                       {'date_creation': DateTime('20141025T15:50:54'),
                        'id': 1177220,
                        'name': 'cloud.iheartcli.com',
                        'state': 'running'},
                       {'date_creation': DateTime('20150728T17:50:56'),
                        'id': 1365951,
                        'name': 'cli.sexy',
                        'state': 'running'}]},
           {'autorenew': None,
            'catalog_name': 'phpmysql_s',
            'console': '1656411@console.dc0.gpaas.net',
            'data_disk_additional_size': 0,
            'datacenter': {'country': 'France',
                           'dc_code': 'FR-SD2',
                           'id': 1,
                           'iso': 'FR',
                           'name': 'Equinix Paris'},
            'datadisk_total_size': 10.0,
            'date_end': DateTime('20160408T00:00:00'),
            'date_end_commitment': None,
            'date_start': DateTime('20130903T22:14:13'),
            'ftp_server': 'sftp.dc0.gpaas.net',
            'git_server': 'git.dc0.gpaas.net',
            'id': 126276,
            'name': 'sap',
            'need_upgrade': False,
            'owner': {'handle': 'PXP561-GANDI', 'id': 2920674},
            'servers': [{'graph_urls': {'vcpu': [''], 'vdi': [''],
                                        'vif': ['']},
                         'id': 126273,
                         'uuid': 195339}],
            'size': 's',
            'snapshot_profile': None,
            'state': 'halted',
            'type': 'phpmysql',
            'user': 1656411,
            'vhosts': [{'date_creation': DateTime('20130903T22:11:54'),
                        'id': 160126,
                        'name': 'aa3e0e26f8.url-de-test.ws',
                        'state': 'running'},
                       {'date_creation': DateTime('20130903T22:24:06'),
                        'id': 160127,
                        'name': 'cloud.cat.lol',
                        'state': 'running'}]},
           {'autorenew': None,
            'catalog_name': 'pythonpgsql_s',
            'console': '1185290@console.dc2.gpaas.net',
            'data_disk_additional_size': 0,
            'datacenter': {'country': 'Luxembourg',
                           'dc_code': 'LU-BI1',
                           'id': 3,
                           'iso': 'LU',
                           'name': 'Bissen'},
            'datadisk_total_size': 10.0,
            'date_end': DateTime('20161125T15:52:56'),
            'date_end_commitment': DateTime('20151118T00:00:00'),
            'date_start': DateTime('20141025T15:52:56'),
            'ftp_server': 'sftp.dc2.gpaas.net',
            'git_server': 'git.dc2.gpaas.net',
            'id': 123456,
            'name': '123456',
            'need_upgrade': False,
            'owner': {'handle': 'PXP561-GANDI', 'id': 2920674},
            'servers': [{'graph_urls': {'vcpu': [''], 'vdi': [''],
                                        'vif': ['']},
                         'id': 1123456,
                         'uuid': 119254}],
            'size': 's',
            'snapshot_profile': None,
            'state': 'running',
            'type': 'pythonpgsql',
            'user': 1185290,
            'vhosts': [{'date_creation': DateTime('20141025T15:50:54'),
                        'id': 2177216,
                        'name': '987832c2b34.testurl.ws',
                        'state': 'running'}]}]

    instances = dict([(paas['id'], paas) for paas in ret])
    return instances[paas_id]


def vhost_list(options):

    ret = [{'cert_id': None,
            'date_creation': DateTime('20130903T22:11:54'),
            'id': 160126,
            'name': 'aa3e0e26f8.url-de-test.ws',
            'paas_id': 126276,
            'state': 'running'},
           {'cert_id': None,
            'date_creation': DateTime('20130903T22:24:06'),
            'id': 160127,
            'name': 'cloud.cat.lol',
            'paas_id': 126276,
            'state': 'running'},
           {'cert_id': None,
            'date_creation': DateTime('20141025T15:50:54'),
            'id': 1177216,
            'name': '187832c2b34.testurl.ws',
            'paas_id': 163744,
            'state': 'running'},
           {'cert_id': None,
            'date_creation': DateTime('20141025T15:50:54'),
            'id': 1177220,
            'name': 'cloud.iheartcli.com',
            'paas_id': 163744,
            'state': 'running'},
           {'cert_id': None,
            'date_creation': DateTime('20150728T17:50:56'),
            'id': 1365951,
            'name': 'cli.sexy',
            'paas_id': 163744,
            'state': 'running'}]

    options.pop('items_per_page', None)

    for fkey in options:
        ret = [paas for paas in ret if paas[fkey] == options[fkey]]

    return ret


def vhost_info(name):
    vhosts = vhost_list({})
    vhosts = dict([(vhost['name'], vhost) for vhost in vhosts])
    return vhosts[name]


def vhost_delete(name):
    return {'id': 200, 'step': 'WAIT', 'name': 'rproxy_update',
            'paas_id': 1177220,
            'date_creation': DateTime('20150728T17:50:56')}


def type_list(options):

    ret = [{'database': 'MySQL',
            'language': 'PHP',
            'name': 'phpmysql'},
           {'database': 'PostgreSQL',
            'language': 'PHP',
            'name': 'phppgsql'},
           {'database': 'PostgreSQL',
            'language': 'Node.js',
            'name': 'nodejspgsql'},
           {'database': 'MongoDB',
            'language': 'Node.js',
            'name': 'nodejsmongodb'},
           {'database': 'MySQL',
            'language': 'Node.js',
            'name': 'nodejsmysql'},
           {'database': 'MongoDB',
            'language': 'PHP',
            'name': 'phpmongodb'},
           {'database': 'MySQL',
            'language': 'Python',
            'name': 'pythonmysql'},
           {'database': 'PostgreSQL',
            'language': 'Python',
            'name': 'pythonpgsql'},
           {'database': 'MongoDB',
            'language': 'Python',
            'name': 'pythonmongodb'},
           {'database': 'MySQL',
            'language': 'Ruby',
            'name': 'rubymysql'},
           {'database': 'PostgreSQL',
            'language': 'Ruby',
            'name': 'rubypgsql'},
           {'database': 'MongoDB',
            'language': 'Ruby',
            'name': 'rubymongodb'}]

    return ret


def update(paas_id, options):
    return {'id': 200, 'step': 'WAIT'}


def vhost_create(options):
    return {'id': 200, 'step': 'WAIT'}


def delete(paas_id):
    return {'id': 200, 'step': 'WAIT'}


def create(options):
    return {'id': 200, 'step': 'WAIT'}


def restart(options):
    return {'id': 200, 'step': 'WAIT'}



# -*- coding: utf-8 -*-

from .base import CommandTestCase
from gandi.cli.commands import account


class AccountTestCase(CommandTestCase):

    def test_info(self):

        result = self.invoke_with_exceptions(account.info, [])

        self.assertEqual(result.output, """\
handle           : PXP561-GANDI
prepaid          : 1337.42 EUR
credits          :
        available: 2335360
        usage    : 633/h
        time left: 0 year(s) 4 month(s) 29 day(s) 17 hour(s)
""")
        self.assertEqual(result.exit_code, 0)



""" Docker namespace commands. """

import os

import click

from gandi.cli.core.cli import cli
from gandi.cli.core.params import pass_gandi


@cli.command()
@click.option('--vm', help='Use given VM for docker connection')
@click.argument('args', nargs=-1)
@pass_gandi
def docker(gandi, vm, args):
    """
    Manage docker instance
    """
    if not [basedir for basedir in os.getenv('PATH', '.:/usr/bin').split(':')
            if os.path.exists('%s/docker' % basedir)]:
        gandi.echo("""'docker' not found in $PATH, required for this command \
to work
See https://docs.docker.com/installation/#installation to install, or use:
    # curl https://get.docker.io/ | sh""")
        return

    if vm:
        gandi.configure(True, 'dockervm', vm)
    else:
        vm = gandi.get('dockervm')

    if not vm:
        gandi.echo("""
No docker vm specified. You can create one:
    $ gandi vm create --hostname docker --image "Ubuntu 14.04 64 bits LTS (HVM)" \\
        --run 'wget -O - https://get.docker.io/ | sh'

Then configure it using:
    $ gandi docker --vm docker ps

Or to both change target vm and spawn a process (note the -- separator):
    $ gandi docker --vm myvm -- run -i -t debian bash
""")  # noqa
        return

    return gandi.docker.handle(vm, args)



""" Vlan namespace commands. """

import click
from IPy import IP

from gandi.cli.core.cli import cli
from gandi.cli.core.utils import (
    output_vlan, output_generic, output_iface, output_ip, output_line,
    DatacenterLimited
)
from gandi.cli.core.params import option, pass_gandi, DATACENTER


@cli.group(name='vlan')
@pass_gandi
def vlan(gandi):
    """Commands related to hosting virtual lan networks."""


@vlan.command()
@click.option('--datacenter', type=DATACENTER, default=None,
              help='Filter by datacenter.')
@click.option('--id', help='Display ids.', is_flag=True)
@click.option('--subnet', help='Display subnets.', is_flag=True)
@click.option('--gateway', help='Display gateway.', is_flag=True)
@pass_gandi
def list(gandi, datacenter, id, subnet, gateway):
    """List vlans."""
    output_keys = ['name', 'state', 'dc']
    if id:
        output_keys.append('id')
    if subnet:
        output_keys.append('subnet')
    if gateway:
        output_keys.append('gateway')

    datacenters = gandi.datacenter.list()

    vlans = gandi.vlan.list(datacenter)
    for num, vlan in enumerate(vlans):
        if num:
            gandi.separator_line()
        output_vlan(gandi, vlan, datacenters, output_keys)

    return vlans


@vlan.command()
@click.option('--ip', help='Display ips.', is_flag=True)
@click.argument('resource')
@pass_gandi
def info(gandi, resource, ip):
    """Display information about a vlan."""
    output_keys = ['name', 'state', 'dc', 'subnet', 'gateway']

    datacenters = gandi.datacenter.list()

    vlan = gandi.vlan.info(resource)

    gateway = vlan['gateway']
    if not ip:
        output_vlan(gandi, vlan, datacenters, output_keys, justify=11)
        return vlan

    gateway_exists = False

    vms = dict([(vm_['id'], vm_) for vm_ in gandi.iaas.list()])
    ifaces = gandi.vlan.ifaces(resource)

    for iface in ifaces:
        for ip in iface['ips']:
            if gateway == ip['ip']:
                gateway_exists = True

    if gateway_exists:
        vlan.pop('gateway')
    else:
        vlan['gateway'] = ("%s don't exists" % gateway if gateway
                           else 'none')

    output_vlan(gandi, vlan, datacenters, output_keys, justify=11)

    output_keys = ['vm', 'bandwidth']
    for iface in ifaces:
        gandi.separator_line()
        output_iface(gandi, iface, datacenters, vms, output_keys,
                     justify=11)
        for ip in iface['ips']:
            output_ip(gandi, ip, None, None, None, ['ip'])
            if gateway == ip['ip']:
                output_line(gandi, 'gateway', 'true', justify=11)

    return vlan


@vlan.command()
@click.option('--background', '--bg', default=False, is_flag=True,
              help='Run command in background mode (default=False).')
@click.option('--force', '-f', is_flag=True,
              help='This is a dangerous option that will cause CLI to continue'
                   ' without prompting. (default=False).')
@click.argument('resource', nargs=-1, required=True)
@pass_gandi
def delete(gandi, background, force, resource):
    """Delete a vlan.

    Resource can be a vlan name or an ID
    """
    output_keys = ['id', 'type', 'step']

    possible_resources = gandi.vlan.resource_list()
    for item in resource:
        if item not in possible_resources:
            gandi.echo('Sorry vlan %s does not exist' % item)
            gandi.echo('Please use one of the following: %s' %
                       possible_resources)
            return

    if not force:
        vlan_info = "'%s'" % ', '.join(resource)
        proceed = click.confirm('Are you sure to delete vlan %s?' %
                                vlan_info)

        if not proceed:
            return

    opers = gandi.vlan.delete(resource, background)
    if background:
        for oper in opers:
            output_generic(gandi, oper, output_keys)

    return opers


@vlan.command()
@click.option('--name', required=True, help='Name of the vlan.')
@option('--datacenter', type=DATACENTER, default='FR-SD5',
        help='Datacenter where the vlan will be spawned.')
@click.option('--subnet', help='The vlan subnet.')
@click.option('--gateway', help='The vlan gateway.')
@click.option('--background', '--bg', default=False, is_flag=True,
              help='Run command in background mode (default=False).')
@pass_gandi
def create(gandi, name, datacenter, subnet, gateway, background):
    """ Create a new vlan """
    try:
        gandi.datacenter.is_opened(datacenter, 'iaas')
    except DatacenterLimited as exc:
        gandi.echo('/!\ Datacenter %s will be closed on %s, '
                   'please consider using another datacenter.' %
                   (datacenter, exc.date))

    result = gandi.vlan.create(name, datacenter, subnet, gateway, background)

    if background:
        gandi.pretty_echo(result)

    return result


@vlan.command()
@click.option('--name', help='Name of the vlan.')
@click.option('--gateway', help='Gateway of the vlan.')
@click.option('--create', default=False, is_flag=True,
              help='If gateway is a vm and does not have any ip in the good '
                   'vlan, we will start by putting an ip in the vlan.')
@option('--bandwidth', type=click.INT, default=102400,
        help="Network bandwidth in kbit/s used to create the VM's ip in this "
             'vlan.')
@click.argument('resource')
@pass_gandi
def update(gandi, resource, name, gateway, create, bandwidth):
    """ Update a vlan

    ``gateway`` can be a vm name or id, or an ip.
    """
    params = {}
    if name:
        params['name'] = name

    vlan_id = gandi.vlan.usable_id(resource)

    try:
        if gateway:
            IP(gateway)
            params['gateway'] = gateway
    except ValueError:
        vm = gandi.iaas.info(gateway)
        ips = [ip for sublist in
               [[ip['ip'] for ip in iface['ips'] if ip['version'] == 4]
                for iface in vm['ifaces']
                if iface['vlan'] and iface['vlan'].get('id') == vlan_id]
               for ip in sublist]

        if len(ips) > 1:
            gandi.echo("This vm has two ips in the vlan, don't know which one"
                       ' to choose (%s)' % (', '.join(ips)))
            return

        if not ips and not create:
            gandi.echo("Can't find '%s' in '%s' vlan" % (gateway, resource))
            return

        if not ips and create:
            gandi.echo('Will create a new ip in this vlan for vm %s' % gateway)
            oper = gandi.ip.create('4', vm['datacenter_id'], bandwidth,
                                   vm['hostname'], resource)

            iface_id = oper['iface_id']
            iface = gandi.iface.info(iface_id)
            ips = [ip['ip'] for ip in iface['ips'] if ip['version'] == 4]

        params['gateway'] = ips[0]

    result = gandi.vlan.update(resource, params)

    return result
